@@ -1,11 +1,15 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997 Marko Macek
+ * Copyright (C) 1997,1998 Marko Macek
  */
 
 #include "icewm.h"
 
+void YFrameControl::setFrame(YFrameWindow *newFrame) {
+    fFrame = newFrame;
+}
+
 YFrameClient::YFrameClient(YWindow *parent, YFrameWindow *frame, Window win): YFrameControl(parent, frame, win) {
     fClient = win ? win : handle();
     fBorder = 0;
@@ -15,45 +19,31 @@ YFrameClient::YFrameClient(YWindow *parent, YFrameWindow *frame, Window win): YF
     fColormap = None;
     fShaped = 0;
     fHints = 0;
+    //fSavedFrameState =
     fSizeHints = XAllocSizeHints();
     fClassHint = XAllocClassHint();
-    fWorkspaces = 1 << app->activeWorkspace();
-    fAllWorkspaces = 0;
     fTransientFor = 0;
 #ifndef NO_MWM_HINTS
     fMwmHints = 0;
 #endif
-    
-    getProtocols();
-    {
-        XTextProperty prop;
 
-        if (XGetWMName(app->display(), handle(), &prop)) {
-            setWindowTitle(prop.value);
-            if (prop.value) XFree(prop.value);
-        }
-        if (XGetWMIconName(app->display(), handle(), &prop)) {
-            setIconTitle(prop.value);
-            if (prop.value) XFree(prop.value);
-        }
-    }
+    getProtocols();
+    getNameHint();
+    getIconNameHint();
     getSizeHints();
     getClassHint();
+    getTransient();
+    getWMHints();
 #ifndef NO_MWM_HINTS
     getMwmHints();
 #endif
-
-    getTransient();
     
-    if ((fHints = XGetWMHints(app->display(), handle())) != 0) {
-    }
 #ifdef SHAPE
     if (shapesSupported) {
         XShapeSelectInput (app->display(), handle(), ShapeNotifyMask);
         queryShape();
     }
 #endif
-    getClientWorkspaces();
     XSaveContext(app->display(),
                  clientWindow(),
                  frame ? frameContext : clientContext,
@@ -64,8 +54,8 @@ YFrameClient::~YFrameClient() {
     XDeleteContext(app->display(),
                    clientWindow(),
                    frame() ? frameContext : clientContext);
-    free(fWindowTitle);
-    free(fIconTitle);
+    FREE(fWindowTitle); fWindowTitle = 0;
+    FREE(fIconTitle); fIconTitle = 0;
     if (fSizeHints) { XFree(fSizeHints); fSizeHints = 0; }
     if (fClassHint) {
         if (fClassHint->res_name) {
@@ -87,13 +77,14 @@ void YFrameClient::getProtocols() {
     Atom *wmp = 0;
     int count;
 
+#ifdef CONFIG_WINLIST
     if (this == windowList) {
         fProtocols = wpDeleteWindow;
         return ;
     }
-    
-    // ???: never remove this one
-    fProtocols = fProtocols & wpDeleteWindow; // = 0;
+#endif
+
+    fProtocols = fProtocols & wpDeleteWindow; // always keep WM_DELETE_WINDOW
 
     if (XGetWMProtocols (app->display(),
                          handle(),
@@ -135,13 +126,13 @@ void YFrameClient::getSizeHints() {
             fSizeHints->min_height = fSizeHints->base_height;
         }
         if (!(fSizeHints->flags & PMaxSize)) {
-            fSizeHints->max_width = app->maxWidth();
-            fSizeHints->max_height = app->maxHeight();
+            fSizeHints->max_width = manager->maxWidth(WinLayerAboveDock);
+            fSizeHints->max_height = manager->maxHeight(WinLayerAboveDock);
         }
         if (fSizeHints->max_width < fSizeHints->min_width)
-            fSizeHints->max_width = app->maxWidth();
+            fSizeHints->max_width = manager->maxWidth(WinLayerAboveDock);
         if (fSizeHints->max_height < fSizeHints->min_height)
-            fSizeHints->max_height = app->maxHeight();
+            fSizeHints->max_height = manager->maxHeight(WinLayerAboveDock);
         
         if (fSizeHints->min_height <= 0)
             fSizeHints->min_height = 1;
@@ -176,7 +167,8 @@ void YFrameClient::getTransient() {
                              handle(),
                              &newTransientFor))
     {
-        if (newTransientFor == app->root()->handle() || /* bug in xfm */
+        if (newTransientFor == manager->handle() || /* bug in xfm */
+            newTransientFor == desktop->handle() ||
             newTransientFor == handle()                 /* bug in fdesign */
             /* !!! TODO: check for recursion */
            )
@@ -194,8 +186,7 @@ void YFrameClient::getTransient() {
     }
 }
 
-
-void YFrameClient::constrainSize(int &w, int &h, int flags) {
+void YFrameClient::constrainSize(int &w, int &h, CARD32 layer, int flags) {
     if (fSizeHints) {
         int wm = fSizeHints->min_width;
         int hm = fSizeHints->min_height;
@@ -255,8 +246,8 @@ void YFrameClient::constrainSize(int &w, int &h, int flags) {
         if (w > wM) w = wM; // maximum size
         if (h > hM) h = hM;
 
-        if (w >= int(app->maxWidth())) w = app->maxWidth();
-        if (h >= int(app->maxHeight())) h = app->maxHeight();
+        if (w >= int(manager->maxWidth(layer))) w = manager->maxWidth(layer);
+        if (h >= int(manager->maxHeight(layer))) h = manager->maxHeight(layer);
 
         w = wb + (w - wb + ((flags & csRound) ? wf / 2 : 0)) / wf * wf;
         h = hb + (h - hb + ((flags & csRound) ? hf / 2 : 0)) / hf * hf;
@@ -320,11 +311,8 @@ void YFrameClient::setFrame(YFrameWindow *newFrame) {
     YFrameControl::setFrame(newFrame);
     if (newFrame == 0) {
         //msg("remove frame");
-        //setWMState(WithdrawnState);
     } else {
         //msg("got frame");
-        getClientWorkspaces();
-        setClientWorkspaces();
     }
     XSaveContext(app->display(),
                  clientWindow(),
@@ -332,24 +320,29 @@ void YFrameClient::setFrame(YFrameWindow *newFrame) {
                  (XPointer)(frame() ? (YWindow *)frame() : (YWindow *)this));
 }
 
-void YFrameClient::setWMState(FrameState state) {
+void YFrameClient::setFrameState(FrameState state) {
     unsigned long arg[2];
 
     arg[0] = (unsigned long) state;
     arg[1] = (unsigned long) None;
 
-    XChangeProperty(app->display(), handle(),
-                    _XA_WM_STATE, _XA_WM_STATE,
-                    32, PropModeReplace,
-                    (unsigned char *)arg, 2);
-
-    if (state == WithdrawnState && phase == phaseRunning)
-        XDeleteProperty(app->display(), handle(), _XA_WM_WORKSPACE);
-    else
-        setClientWorkspaces();
+    if (state == WithdrawnState) {
+        if (phase != phaseRestart && phase != phaseShutdown) {
+            MSG(("deleting window properties id=%lX", handle()));
+            XDeleteProperty(app->display(), clientWindow(), _XA_WIN_WORKSPACE);
+            XDeleteProperty(app->display(), clientWindow(), _XA_WIN_LAYER);
+            XDeleteProperty(app->display(), clientWindow(), _XA_WIN_STATE);
+            XDeleteProperty(app->display(), clientWindow(), _XA_WM_STATE);
+        }
+    } else {
+        XChangeProperty(app->display(), handle(),
+                        _XA_WM_STATE, _XA_WM_STATE,
+                        32, PropModeReplace,
+                        (unsigned char *)arg, 2);
+    }
 }
 
-FrameState YFrameClient::getWMState() {
+FrameState YFrameClient::getFrameState() {
     FrameState st = WithdrawnState;
     Atom type;
     int format;
@@ -359,141 +352,74 @@ FrameState YFrameClient::getWMState() {
     if (XGetWindowProperty(app->display(), handle(),
                            _XA_WM_STATE, 0, 3, False, _XA_WM_STATE,
                            &type, &format, &nitems, &lbytes,
-                           &propdata) == Success)
-        if (propdata) {
-            st = *(long *)propdata;
-            XFree(propdata);
-        }
-    return st;
-}
-
-void YFrameClient::configureClient(const XConfigureRequestEvent &configureRequest) {
-    int cx, cy, cwidth, cheight;
-
-    setBorder((configureRequest.value_mask & CWBorderWidth) ? configureRequest.border_width : border());
-    cx = (configureRequest.value_mask & CWX) ? configureRequest.x + DO_BORDER(border()) : x();
-    cy = (configureRequest.value_mask & CWY) ? configureRequest.y + DO_BORDER(border()) : y();
-    cwidth = (configureRequest.value_mask & CWWidth) ? configureRequest.width : width();
-    cheight = (configureRequest.value_mask & CWHeight) ? configureRequest.height : height();
-
-    //constrainSize(cwidth, cheight);
-
-    setGeometry(cx,
-                cy,
-                cwidth,
-                cheight);
-
-    if (configureRequest.value_mask & CWStackMode) {
-        YFrameWindow *sibling;
-        XWindowChanges xwc;
-
-        if ((configureRequest.value_mask & CWSibling) &&
-            XFindContext(app->display(),
-                         configureRequest.above,
-                         frameContext,
-                         (XPointer *)&sibling) == 0)
-            xwc.sibling = sibling->handle();
-        else
-            xwc.sibling = configureRequest.above;
-
-        xwc.stack_mode = configureRequest.detail;
-
-        XConfigureWindow (app->display(),
-                          handle(),
-                          configureRequest.value_mask & (CWSibling | CWStackMode),
-                          &xwc);
-
+                           &propdata) == Success && propdata)
+    {
+        st = *(long *)propdata;
+        XFree(propdata);
     }
+    return st;
 }
 
 void YFrameClient::handleUnmap(const XUnmapEvent &unmap) {
     MSG(("Unmap: unmapped %d visible %d", unmapped(), visible()));
     if (!unmapped()) {
         MSG(("UnmapWindow"));
+#ifdef NEED_GRAB
         XGrabServer(app->display());
         XSync(app->display(), 0);
+#endif
         XEvent ev;
         if (XCheckTypedWindowEvent(app->display(), unmap.window, DestroyNotify, &ev)) {
-            app->destroyedWindow(unmap.window);
+            manager->destroyedWindow(unmap.window);
             XUngrabServer(app->display());
             return ; // gets destroyed
         } else {
-            setWMState(WithdrawnState);
-            app->unmapWindow(unmap.window);
+            manager->unmanageWindow(unmap.window, false);
         }
+#ifdef NEED_GRAB
         XUngrabServer(app->display());
+#endif
     }
     YWindow::handleUnmap(unmap);
 }
 
 void YFrameClient::handleProperty(const XPropertyEvent &property) {
-    int propDelete = (property.state == PropertyDelete) ? 1 : 0;
-        
     switch (property.atom) {
     case XA_WM_NAME:
-        {
-            XTextProperty prop;
-
-            if (propDelete)
-                setWindowTitle(0);
-            else if (XGetWMName(app->display(), handle(), &prop)) {
-                setWindowTitle(prop.value);
-		if (prop.value) XFree(prop.value);
-            } else {
-                setWindowTitle(0);
-            }
-        }
+        getNameHint();
         break;
-        
     case XA_WM_ICON_NAME:
-        {
-            XTextProperty prop;
-
-            if (propDelete)
-                setIconTitle(0);
-            else if (XGetWMIconName(app->display(), handle(), &prop)) {
-                setIconTitle(prop.value);
-		if (prop.value) XFree(prop.value);
-            } else {
-                setIconTitle(0);
-            }
-        }
+        getIconNameHint();
         break;
-
     case XA_WM_CLASS:
         getClassHint();
         if (frame())
             frame()->getFrameHints();
         break;
-        
     case XA_WM_HINTS:
-        if (fHints)
-            XFree(fHints);
-        if ((fHints = XGetWMHints(app->display(), handle())) != 0) {
-        }
+        getWMHints();
         break;
-        
     case XA_WM_NORMAL_HINTS:
         getSizeHints();
         break;
-        
     case XA_WM_TRANSIENT_FOR:
         getTransient();
         break;
-       
     default:
         if (property.atom == _XA_WM_PROTOCOLS)
             getProtocols();
+        else if (property.atom == _XA_KWM_WIN_ICON) {
+            if (frame())
+                frame()->updateIcon();
+        } else if (property.atom == _XA_WIN_ICONS) {
+            if (frame())
+                frame()->updateIcon();
+        }
 #ifndef NO_MWM_HINTS
         else if (property.atom == _XATOM_MWM_HINTS) {
             getMwmHints();
-            if (frame()) {
-                frame()->getFrameHints();
-                frame()->configure(frame()->x(),
-                                   frame()->y(),
-                                   frame()->width(),
-                                   frame()->height());
-            }
+            if (frame())
+                frame()->updateMwmHints();
         }
 #endif
         break;
@@ -512,7 +438,7 @@ void YFrameClient::handleDestroyWindow(const XDestroyWindowEvent &destroyWindow)
     YWindow::handleDestroyWindow(destroyWindow);
 
     if (destroyed())
-        app->destroyedWindow(destroyWindow.window);
+        manager->destroyedWindow(destroyWindow.window);
 }
 
 #ifdef SHAPE
@@ -528,23 +454,61 @@ void YFrameClient::handleShapeNotify(const XShapeEvent &shape) {
 #endif
 
 void YFrameClient::setWindowTitle(unsigned char *aWindowTitle) {
-    free(fWindowTitle);
+    FREE(fWindowTitle);
     fWindowTitle = aWindowTitle ? (unsigned char *)strdup((char *)aWindowTitle) : 0;
     if (frame())
         frame()->updateTitle();
 }
 
+#ifdef I18N
+void YFrameClient::setWindowTitle(XTextProperty  *prop) {
+    Status status;
+    char **cl;
+    int n;
+    if (!prop->value || prop->encoding == XA_STRING) {
+	setWindowTitle((unsigned char *)prop->value);
+	return;
+    }
+    status = XmbTextPropertyToTextList (app->display(), prop, &cl, &n);
+    if (status >= Success && n > 0 && cl[0]) {
+	setWindowTitle((unsigned char *)cl[0]);
+	XFreeStringList(cl);
+    } else {
+	setWindowTitle((unsigned char *)prop->value);
+    }
+}
+#endif
+
 void YFrameClient::setIconTitle(unsigned char *aIconTitle) {
-    free(fIconTitle);
+    FREE(fIconTitle);
     fIconTitle = aIconTitle ? (unsigned char *)strdup((char *)aIconTitle) : 0;
     if (frame())
         frame()->updateIconTitle();
 }
 
+#ifdef I18N
+void YFrameClient::setIconTitle(XTextProperty  *prop) {
+    Status status;
+    char **cl;
+    int n;
+    if (!prop->value || prop->encoding == XA_STRING) {
+	setIconTitle((unsigned char *)prop->value);
+	return;
+    }
+    status = XmbTextPropertyToTextList (app->display(), prop, &cl, &n);
+    if (status >= Success && n > 0 && cl[0]) {
+	setIconTitle((unsigned char *)cl[0]);
+	XFreeStringList(cl);
+    } else {
+	setIconTitle((unsigned char *)prop->value);
+    }
+}
+#endif
+
 void YFrameClient::setColormap(Colormap cmap) {
     fColormap = cmap;
-    if (frame() && app->colormapWindow() == frame())
-        app->installColormap(cmap);
+    if (frame() && manager->colormapWindow() == frame())
+        manager->installColormap(cmap);
 }
 
 #ifdef SHAPE
@@ -564,141 +528,10 @@ void YFrameClient::queryShape() {
 }
 #endif
 
-#ifndef NO_MWM_HINTS
-void YFrameClient::getMwmHints() {
-    int retFormat;
-    Atom retType;
-    unsigned long retCount, remain;
-
-    if (fMwmHints) {
-        XFree(fMwmHints);
-        fMwmHints = 0;
-    }
-    if (XGetWindowProperty(app->display(), handle(),
-                           _XATOM_MWM_HINTS, 0L, 20L, False, _XATOM_MWM_HINTS,
-                           &retType, &retFormat, &retCount,
-                           &remain,(unsigned char **)&fMwmHints) == Success)
-        if (retCount >= PROP_MWM_HINTS_ELEMENTS)
-            return;
-    fMwmHints = 0;
-}
-
-CARD32 YFrameClient::mwmFunctions() {
-    CARD32 functions = ~0U;
-    
-    if (fMwmHints && (fMwmHints->flags & MWM_HINTS_FUNCTIONS)) {
-        if (fMwmHints->functions & MWM_FUNC_ALL)
-            functions = ~fMwmHints->functions;
-        else
-            functions = fMwmHints->functions;
-    }
-    functions &= (MWM_FUNC_RESIZE | MWM_FUNC_MOVE |
-                  MWM_FUNC_MINIMIZE | MWM_FUNC_MAXIMIZE |
-                  MWM_FUNC_CLOSE);
-    return functions;
-}
-
-CARD32 YFrameClient::mwmDecors() {
-    CARD32 decors = ~0U;
-    CARD32 func = mwmFunctions();
-    
-    if (fMwmHints && (fMwmHints->flags & MWM_HINTS_DECORATIONS)) {
-        if (fMwmHints->decorations & MWM_FUNC_ALL)
-            decors = ~fMwmHints->decorations;
-        else
-            decors = fMwmHints->decorations;
-    }
-    decors &= (MWM_DECOR_BORDER | MWM_DECOR_RESIZEH |
-               MWM_DECOR_TITLE | MWM_DECOR_MENU |
-               MWM_DECOR_MINIMIZE | MWM_DECOR_MAXIMIZE);
-
-    decors &=
-        ~(((func & MWM_FUNC_RESIZE) ? 0 : MWM_DECOR_RESIZEH) |
-          ((func & MWM_FUNC_MINIMIZE) ? 0 : MWM_DECOR_MINIMIZE) |
-          ((func & MWM_FUNC_MAXIMIZE) ? 0 : MWM_DECOR_MAXIMIZE));
-
-    return decors;
-}
-#endif
-
-void YFrameClient::setClientWorkspaces() {
-    INT32 wk[MAXWORKSPACES];
-    int nwk = 0;
-
-    if (fAllWorkspaces)
-        wk[nwk++] = -1;
-    else
-        for (int i = 0; i < workspaceCount; i++)
-            if (workspaces() & (1 << i))
-                wk[nwk++] = i;
-
-    XChangeProperty(app->display(),
-                    handle(),
-                    _XA_WM_WORKSPACE,
-                    XA_INTEGER,
-                    32, PropModeReplace,
-                    (unsigned char *)wk, nwk);
-}
-
-void YFrameClient::getClientWorkspaces() {
-    Atom r_type;
-    int r_format;
-    unsigned long count;
-    unsigned long bytes_remain;
-    unsigned char *prop;
-
-    fWorkspaces = 1 << app->activeWorkspace();
-    fAllWorkspaces = 0;
-    if (XGetWindowProperty(app->display(),
-                           handle(),
-                           _XA_WM_WORKSPACE,
-                           0, 1, False, XA_INTEGER,
-                           &r_type, &r_format,
-                           &count, &bytes_remain, &prop) == Success && prop)
-    {
-        if (r_type == XA_INTEGER && r_format == 32 && count == 1U) {
-            INT32 *wsn = (INT32 *)prop;
-            if (wsn[0] == -1)
-                fAllWorkspaces = 1;
-            else if (wsn[0] >= 0 && wsn[0] < workspaceCount) {
-                unsigned long ws = (1 << wsn[0]);
-                fWorkspaces = ws ? ws : 1;
-            }
-        }
-        XFree(prop);
-    }
-    MSG(("%s all: %d, workspaces: %lX\n", windowTitle(), fAllWorkspaces, fWorkspaces));
-}
-
-void YFrameClient::setWorkspaces(unsigned long workspaces) {
-    if (workspaces != fWorkspaces) {
-        fWorkspaces = workspaces;
-        setClientWorkspaces();
-        if (frame()) {
-            if (frame()->visibleOn(app->activeWorkspace()))
-                frame()->workspaceShow();
-            else
-                frame()->workspaceHide();
-        }
-    }
-}
-
-void YFrameClient::setAllWorkspaces(int allWorkspaces) {
-    if (fAllWorkspaces != allWorkspaces) {
-        fAllWorkspaces  = allWorkspaces;
-        setClientWorkspaces();
-        if (frame()) {
-            if (frame()->visibleOn(app->activeWorkspace()))
-                frame()->workspaceShow();
-            else
-                frame()->workspaceHide();
-        }
-    }
-}
-
 YClientContainer::YClientContainer(YWindow *parent, YFrameWindow *frame)
 :YFrameControl(parent, frame)
 {
+    setStyle(wsManager);
     haveButtonGrab = 0;
 }
 
@@ -712,12 +545,25 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
         if (haveButtonGrab)
             MSG(("buttonGrab"));
 
-        if (!(button.state & ControlMask)) {
+        if (!(button.state & ControlMask) &&
+            (!useMouseWheel || (button.button != 4 && button.button != 5)))
+        {
             if (focusOnClickClient)
                 frame()->activate();
             if (raiseOnClickClient)
                 frame()->wmRaise();
         }
+#if 0
+        if (button.button == 3 &&
+            ((button.state &
+              (ControlMask | ShiftMask | app->AltMask) ==
+              (ControlMask | ShiftMask))))
+        {
+            frame()->wmMove();
+            XAllowEvents(app->display(), AsyncPointer, CurrentTime);
+            return ;
+        }
+#endif
         if (passFirstClickToClient)
             XAllowEvents(app->display(), ReplayPointer, CurrentTime);
         else
@@ -734,6 +580,7 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
 
 // !!! fix ('not on top'  should be  'can be raised').
 //     the difference is when we have transients and explicitFocus
+//     also there is the difference with layers
 
 void YClientContainer::grabButtons() {
     if (!haveButtonGrab &&
@@ -773,25 +620,10 @@ void YClientContainer::handleConfigureRequest(const XConfigureRequestEvent &conf
 
 void YClientContainer::handleMapRequest(const XMapRequestEvent &mapRequest) {
     if (mapRequest.window == frame()->client()->clientWindow()) {
-        if (frame()->minimized())
-            frame()->wmMinimize();
-        if (frame()->rolledup())
-            frame()->wmRollup();
-        frame()->changeState(NormalState);
-        frame()->focusOnMap();
-    }
-}
-
-void YFrameClient::handleMap(const XMapEvent &map) {
-    YWindow::handleMap(map);
-    if (map.window == clientWindow() && frame()) {
-        if (windowList && windowList->handle() != map.window)
-            return ;
-        if (frame()->minimized())
-            frame()->wmMinimize();
-        if (frame()->rolledup())
-            frame()->wmRollup();
-        frame()->changeState(NormalState);
+        frame()->setState(WinStateMinimized |
+                          WinStateHidden |
+                          WinStateRollup,
+                          0);
         frame()->focusOnMap();
     }
 }
@@ -799,15 +631,319 @@ void YFrameClient::handleMap(const XMapEvent &map) {
 void YClientContainer::handleCrossing(const XCrossingEvent &crossing) {
     if (frame() && pointerColormap) {
         if (crossing.type == EnterNotify)
-            app->setColormapWindow(frame());
-#ifndef SLOPPY_FOCUS
+            manager->setColormapWindow(frame());
         else if (crossing.type == LeaveNotify &&
                  crossing.detail != NotifyInferior &&
                  crossing.mode == NotifyNormal &&
-                 app->colormapWindow() == frame())
+                 manager->colormapWindow() == frame())
         {
-            app->setColormapWindow(0);
+            manager->setColormapWindow(0);
         }
-#endif
     }
 }
+
+void YFrameClient::handleClientMessage(const XClientMessageEvent &message) {
+    if (message.message_type == _XA_WM_CHANGE_STATE) {
+        YFrameWindow *frame = manager->getFrame(message.window);
+
+        if (message.data.l[0] == IconicState) {
+            if (frame && !(frame->isMinimized() || frame->isRollup()))
+                frame->wmMinimize();
+        } else if (message.data.l[0] == NormalState) {
+            if (frame)
+                frame->setState(WinStateHidden |
+                                WinStateRollup |
+                                WinStateMinimized, 0);
+        } // !!! handle WithdrawnState if needed
+
+    } else if (message.message_type == _XA_WIN_WORKSPACE) {
+        if (frame())
+            frame()->setWorkspace(message.data.l[0]);
+        else
+            setWinWorkspaceHint(message.data.l[0]);
+    } else if (message.message_type == _XA_WIN_LAYER) {
+        if (frame())
+            frame()->setLayer(message.data.l[0]);
+        else
+            setWinLayerHint(message.data.l[0]);
+    } else if (message.message_type == _XA_WIN_STATE) {
+        if (frame())
+            frame()->setState(message.data.l[0], message.data.l[1]);
+        else
+            setWinStateHint(message.data.l[0], message.data.l[1]);
+    } else
+        YWindow::handleClientMessage(message);
+}
+
+void YFrameClient::getNameHint() {
+    XTextProperty prop;
+    
+    if (XGetWMName(app->display(), handle(), &prop)) {
+#ifndef I18N
+        setWindowTitle(prop.value);
+#else
+        setWindowTitle(&prop);
+#endif
+        if (prop.value) XFree(prop.value);
+    } else {
+        setWindowTitle((unsigned char *)0);
+    }
+}
+
+void YFrameClient::getIconNameHint() {
+    XTextProperty prop;
+    
+    if (XGetWMIconName(app->display(), handle(), &prop)) {
+#ifndef I18N
+        setIconTitle(prop.value);
+#else
+        setIconTitle(&prop);
+#endif
+        if (prop.value) XFree(prop.value);
+    } else {
+        setIconTitle((unsigned char *)0);
+    }
+}
+
+void YFrameClient::getWMHints() {
+    if (fHints)
+        XFree(fHints);
+    fHints = XGetWMHints(app->display(), handle());
+}
+
+#ifndef NO_MWM_HINTS
+void YFrameClient::getMwmHints() {
+    int retFormat;
+    Atom retType;
+    unsigned long retCount, remain;
+
+    if (fMwmHints) {
+        XFree(fMwmHints);
+        fMwmHints = 0;
+    }
+    if (XGetWindowProperty(app->display(), handle(),
+                           _XATOM_MWM_HINTS, 0L, 20L, False, _XATOM_MWM_HINTS,
+                           &retType, &retFormat, &retCount,
+                           &remain,(unsigned char **)&fMwmHints) == Success && fMwmHints)
+        if (retCount >= PROP_MWM_HINTS_ELEMENTS)
+            return;
+        else
+            XFree(fMwmHints);
+    fMwmHints = 0;
+}
+
+CARD32 YFrameClient::mwmFunctions() {
+    CARD32 functions = ~0U;
+    
+    if (fMwmHints && (fMwmHints->flags & MWM_HINTS_FUNCTIONS)) {
+        if (fMwmHints->functions & MWM_FUNC_ALL)
+            functions = ~fMwmHints->functions;
+        else
+            functions = fMwmHints->functions;
+    }
+    functions &= (MWM_FUNC_RESIZE | MWM_FUNC_MOVE |
+                  MWM_FUNC_MINIMIZE | MWM_FUNC_MAXIMIZE |
+                  MWM_FUNC_CLOSE);
+    return functions;
+}
+
+CARD32 YFrameClient::mwmDecors() {
+    CARD32 decors = ~0U;
+    CARD32 func = mwmFunctions();
+    
+    if (fMwmHints && (fMwmHints->flags & MWM_HINTS_DECORATIONS)) {
+        if (fMwmHints->decorations & MWM_FUNC_ALL)
+            decors = ~fMwmHints->decorations;
+        else
+            decors = fMwmHints->decorations;
+    }
+    decors &= (MWM_DECOR_BORDER | MWM_DECOR_RESIZEH |
+               MWM_DECOR_TITLE | MWM_DECOR_MENU |
+               MWM_DECOR_MINIMIZE | MWM_DECOR_MAXIMIZE);
+
+    /// !!! add disabled buttons
+    decors &=
+        ~(/*((func & MWM_FUNC_RESIZE) ? 0 : MWM_DECOR_RESIZEH) |*/
+          ((func & MWM_FUNC_MINIMIZE) ? 0 : MWM_DECOR_MINIMIZE) |
+          ((func & MWM_FUNC_MAXIMIZE) ? 0 : MWM_DECOR_MAXIMIZE));
+
+    return decors;
+}
+#endif
+
+bool YFrameClient::getKwmIcon(int *count, Pixmap **pixmap) {
+    Atom r_type;
+    int r_format;
+    unsigned long nitems;
+    unsigned long bytes_remain;
+    unsigned char *prop;
+    
+    if (XGetWindowProperty(app->display(), handle(),
+                           _XA_KWM_WIN_ICON, 0, 2, False, _XA_KWM_WIN_ICON,
+                           &r_type, &r_format, &nitems, &bytes_remain,
+                           &prop) == Success && prop)
+    {
+        if (r_format == 32 &&
+            r_type == _XA_KWM_WIN_ICON &&
+            nitems == 2)
+        {
+            *count = nitems;
+            *pixmap = (Pixmap *)prop;
+
+            if (fHints)
+                XFree(fHints);
+            if ((fHints = XGetWMHints(app->display(), handle())) != 0) {
+            }
+            return true;
+        } else {
+            XFree(prop);
+        }
+    }
+    return false;
+}
+
+bool YFrameClient::getWinIcons(Atom *type, int *count, CARD32 **elem) {
+    Atom r_type;
+    int r_format;
+    unsigned long nitems;
+    unsigned long bytes_remain;
+    unsigned char *prop;
+    
+    if (XGetWindowProperty(app->display(), handle(),
+                           _XA_WIN_ICONS, 0, 4096, False, AnyPropertyType,
+                           &r_type, &r_format, &nitems, &bytes_remain,
+                           &prop) == Success && prop)
+    {
+        if (r_format == 32 && nitems > 0) {
+
+            if (r_type == _XA_WIN_ICONS ||
+                r_type == XA_PIXMAP) // for compatibility, obsolete, (will be removed?)
+            {
+                *type = r_type;
+                *count = nitems;
+                *elem = (CARD32 *)prop;
+                return true;
+            }
+        }
+        XFree(prop);
+    }
+    return false;
+}
+
+void YFrameClient::setWinWorkspaceHint(CARD32 wk) {
+    XChangeProperty(app->display(),
+                    handle(),
+                    _XA_WIN_WORKSPACE,
+                    XA_CARDINAL,
+                    32, PropModeReplace,
+                    (unsigned char *)&wk, 1);
+}
+
+bool YFrameClient::getWinWorkspaceHint(CARD32 *workspace) {
+    Atom r_type;
+    int r_format;
+    unsigned long count;
+    unsigned long bytes_remain;
+    unsigned char *prop;
+
+    if (XGetWindowProperty(app->display(),
+                           handle(),
+                           _XA_WIN_WORKSPACE,
+                           0, 1, False, XA_CARDINAL,
+                           &r_type, &r_format,
+                           &count, &bytes_remain, &prop) == Success && prop)
+    {
+        if (r_type == XA_CARDINAL && r_format == 32 && count == 1U) {
+            CARD32 ws = *(CARD32 *)prop;
+            if (ws < workspaceCount) {
+                *workspace = ws;
+                XFree(prop);
+                return true;
+            }
+        }
+        XFree(prop);
+    }
+    return false;
+}
+
+void YFrameClient::setWinLayerHint(CARD32 layer) {
+    XChangeProperty(app->display(),
+                    handle(),
+                    _XA_WIN_LAYER,
+                    XA_CARDINAL,
+                    32, PropModeReplace,
+                    (unsigned char *)&layer, 1);
+}
+
+bool YFrameClient::getWinLayerHint(CARD32 *layer) {
+    Atom r_type;
+    int r_format;
+    unsigned long count;
+    unsigned long bytes_remain;
+    unsigned char *prop;
+
+    if (XGetWindowProperty(app->display(),
+                           handle(),
+                           _XA_WIN_LAYER,
+                           0, 1, False, XA_CARDINAL,
+                           &r_type, &r_format,
+                           &count, &bytes_remain, &prop) == Success && prop)
+    {
+        if (r_type == XA_CARDINAL && r_format == 32 && count == 1U) {
+            CARD32 l = *(CARD32 *)prop;
+            if (l < WinLayerCount) {
+                *layer = l;
+                XFree(prop);
+                return true;
+            }
+        }
+        XFree(prop);
+    }
+    return false;
+}
+
+bool YFrameClient::getWinStateHint(CARD32 *mask, CARD32 *state) {
+    Atom r_type;
+    int r_format;
+    unsigned long count;
+    unsigned long bytes_remain;
+    unsigned char *prop;
+
+    if (XGetWindowProperty(app->display(),
+                           handle(),
+                           _XA_WIN_STATE,
+                           0, 2, False, XA_CARDINAL,
+                           &r_type, &r_format,
+                           &count, &bytes_remain, &prop) == Success && prop)
+    {
+        MSG(("got state"));
+        if (r_type == XA_CARDINAL && r_format == 32 && count == 2U) {
+            CARD32 m = ((CARD32 *)prop)[0];
+            CARD32 s = ((CARD32 *)prop)[1];
+
+            *state = s;
+            *mask = m;
+            XFree(prop);
+            return true;
+        }
+        MSG(("bad state"));
+        XFree(prop);
+    }
+    return false;
+}
+
+void YFrameClient::setWinStateHint(CARD32 mask, CARD32 state) {
+    CARD32 s[2];
+
+    s[0] = mask;
+    s[1] = state;
+
+    MSG(("set state=%lX mask=%lX", state, mask));
+
+    XChangeProperty(app->display(),
+                    handle(),
+                    _XA_WIN_STATE,
+                    XA_CARDINAL,
+                    32, PropModeReplace,
+                    (unsigned char *)&s, 2);
+}
