@@ -3,7 +3,7 @@
  *
  * If compiled, it can be enabled/disabled in the .rc file.
  *
- * ICEWM assumes the following keyboard mappings:
+ * IceWM assumes the following keyboard mappings:
  *
  * xmodmap - <<EOF
  * ! Left Window key
@@ -15,13 +15,12 @@
  * EOF
  *
  */
-#define SUPPORT_WIN95KBD
-
 #define MAIL_SPOOL "/var/spool/mail/"       /* '/' terminated ! */
 
 /** Use Linux 2.0 Penguin as start button */
 #ifndef START_PIXMAP
 #define START_PIXMAP "linux.xpm"
+//#define START_PIXMAP "bsd-daemon.xpm"
 //#define START_PIXMAP "start.xpm"
 //#define START_PIXMAP "xfree86os2.xpm"
 #endif
@@ -41,13 +40,30 @@
 /* experimental: broadcast various gui events (for icesound) */
 #undef CONFIG_GUIEVENTS
 
-/* TODO */
-#define CONFIG_MAILBOX_STATUS
+/* tooltips */
+#define CONFIG_TOOLTIP
+
+/* taskbar */
 #define CONFIG_TASKBAR
+
+/* mailbox */
+#define CONFIG_MAILBOX
+
+/* clock */
+#define CONFIG_CLOCK
+
+/* start menu */
+#define CONFIG_START
+
+/* window list (not menu) */
 #define CONFIG_WINLIST
+
+/* TODO */
 #define CONFIG_WORKSPACES
 #define CONFIG_MWMHINTS
 #define CONFIG_APPICONS
+#define CONFIG_DESKTOP_ICONS
+#define CONFIG_CLOCK_PRETTY
 
 #define CONFIG_DEFAULT_LOOK lookNice
 #define CONFIG_DEFAULT_THEME "nice/default.theme"
@@ -58,5 +74,7 @@
 #define CONFIG_LOOK_WARP4
 #define CONFIG_LOOK_MOTIF
 #define CONFIG_LOOK_PIXMAP
+#define CONFIG_LOOK_METAL
+#define CONFIG_LOOK_GTK
 
 #define PRETTY_CLOCK
