@@ -1,25 +1,35 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997 Marko Macek
+ * Copyright (C) 1997,1998 Marko Macek
  */
 
 #include "icewm.h"
 
-void YApplication::popup(YWindow *forWindow, YPopupWindow *popup) {
+YColor *menuBg = 0;
+YColor *menuItemFg = 0;
+YColor *activeMenuItemBg = 0;
+YColor *activeMenuItemFg = 0;
+YColor *disabledMenuItemFg = 0;
+
+YFont *menuFont = 0;
+
+bool YApplication::popup(YWindow *forWindow, YPopupWindow *popup) {
     assert(popup != 0);
     if (fPopup == 0) {
         Cursor changePointer = (popup->popupFlags() & YPopupWindow::pfNoPointerChange) ? None : rightPointer;
         
         if (!grabEvents(forWindow ? forWindow : popup, changePointer,
-                        ButtonPressMask | ButtonReleaseMask | ButtonMotionMask))
+                        ButtonPressMask | ButtonReleaseMask |
+                        (menuMouseTracking ? PointerMotionMask : ButtonMotionMask)))
         {
             popup->cancelPopup();
-            return ;
+            return false;
         }
     }
     popup->setPrevPopup(fPopup);
     fPopup = popup;
+    return true;
 }
 
 void YApplication::popdown(YPopupWindow *popdown) {
@@ -57,22 +67,29 @@ void YPopupWindow::activatePopup() {
 void YPopupWindow::deactivatePopup() {
 }
 
-void YPopupWindow::popup(YWindow *forWindow, CommandListener *commandListener, unsigned int flags) {
+bool YPopupWindow::popup(YWindow *forWindow,
+                         CommandListener *commandListener,
+                         PopDownListener *popDown,
+                         unsigned int flags)
+{
     fPrevPopup = 0;
     fFlags = flags;
     fForWindow = forWindow;
     fCommandListener = commandListener;
-    if (fCommandListener)
-        fCommandListener->setPopupActive(this);
+    fPopDownListener = popDown;
 
     raise();
     show();
 
     app->popup(forWindow, this);
     activatePopup();
+    return true;
 }
 
-void YPopupWindow::popup(YWindow *forWindow, CommandListener *commandListener, int x, int y, int x_delta, int y_delta, unsigned int flags) {
+bool YPopupWindow::popup(YWindow *forWindow,
+                         CommandListener *commandListener,
+                         PopDownListener *popDown,
+                         int x, int y, int x_delta, int y_delta, unsigned int flags) {
 
     if ((flags & pfPopupMenu) &&
         (wmLook == lookWarp3 || wmLook == lookWarp4))
@@ -85,18 +102,18 @@ void YPopupWindow::popup(YWindow *forWindow, CommandListener *commandListener, i
     sizePopup();
 
     /* !!! FIX this to maximize visible area */
-    if ((x + width() > app->root()->width()) || (fFlags & pfFlipHorizontal))
+    if ((x + width() > desktop->width()) || (fFlags & pfFlipHorizontal))
         if (fFlags & (pfCanFlipHorizontal | pfFlipHorizontal)) {
             x -= width() + x_delta;
             fFlags |= pfFlipHorizontal;
         } else
-            x = app->root()->width() - width();
-    if ((y + height() > app->root()->height()) || (fFlags & pfFlipVertical))
+            x = desktop->width() - width();
+    if ((y + height() > desktop->height()) || (fFlags & pfFlipVertical))
         if (fFlags & (pfCanFlipVertical | pfFlipVertical)) {
             y -= height() + y_delta;
             fFlags |= pfFlipVertical;
         } else
-            y = app->root()->height() - height();
+            y = desktop->height() - height();
     if (x < 0)
         if (fFlags & pfCanFlipHorizontal)
             x += width() + x_delta;
@@ -108,14 +125,28 @@ void YPopupWindow::popup(YWindow *forWindow, CommandListener *commandListener, i
         else
             y = 0;
 
+    if (forWindow == 0) {
+        if ((x + width() > desktop->width()))
+            x = desktop->width() - width();
+
+        if (x < 0)
+            x = 0;
+
+        if ((y + height() > desktop->height()))
+            y = desktop->height() - height();
+
+        if (y < 0)
+            y = 0;
+    }
+
     setPosition(x, y);
 
-    popup(forWindow, commandListener, fFlags);
+    return popup(forWindow, commandListener, popDown, fFlags);
 }
 
 void YPopupWindow::popdown() {
-    if (fCommandListener)
-        fCommandListener->setPopupActive(0);
+    if (fPopDownListener)
+        fPopDownListener->handlePopDown(this);
     
     deactivatePopup();
     app->popdown(this);
@@ -142,8 +173,8 @@ void YPopupWindow::finishPopup(WMCommand command, void *context, unsigned int mo
 
     if (cmd)
         cmd->handleCommand(command, context, modifiers);
-    else
-        app->handleCommand(command, context, modifiers);
+    else //!!!
+        fprintf(stderr, "no menu listener.\n");
 }
 
 bool YPopupWindow::handleKey(const XKeyEvent &/*key*/) {
@@ -206,60 +237,7 @@ YMenu::~YMenu() {
 
     for (int i = 0; i < fItemCount; i++)
         delete fItems[i];
-    free(fItems);
-}
-
-void YMenu::sizePopup() {
-    int width = 2;
-    int height = 2 + 3;
-    int fontHeight = menuFont->height() + 1;
-    int maxWidth = 0;
-    int maxName = 0;
-    int maxParam = 0;
-    unsigned int maxh = 16;
-
-    for (int i = 0; i < itemCount(); i++) {
-        if (item(i)->command() == cmdSeparator) {
-            height += 4;
-        } else {
-            int tw = 0;
-            
-            char *name = item(i)->name();
-            char *param = item(i)->param();
-            
-            if (name) {
-                tw = XTextWidth(menuFont->getFontStruct(), name, strlen(name));
-                if (tw >= maxName)
-                    maxName = tw;
-            }
-
-            if (param) {
-                tw = XTextWidth(menuFont->getFontStruct(), param, strlen(param));
-                if (tw >= maxParam)
-                    maxParam = tw;
-            } else if (item(i)->submenu()) {
-                tw = fontHeight + 2;
-                if (tw >= maxParam)
-                    maxParam = tw;
-            }
-
-            unsigned int h = fontHeight;
-
-            if (item(i)->getPixmap() && item(i)->getPixmap()->height() > h)
-                h = item(i)->getPixmap()->height();
-
-            if (h > maxh)
-                maxh = h;
-
-            height += h + 3;
-        }
-    }
-
-    maxWidth = maxName + maxParam + (maxParam ? 8 : 0);
-
-    width += 3 + maxh + 2 + 2 + maxWidth + 4 + 4;
-    setSize(width, height);
-    paramPos = maxName;
+    FREE(fItems);
 }
 
 void YMenu::activatePopup() {
@@ -288,68 +266,6 @@ void YMenu::donePopup(YPopupWindow *popup) {
     }
 }
 
-void YMenu::paintItems() {
-    Graphics &g = getGraphics();
-    int yy = 2;
-    for (int i = 0; i < itemCount(); i++)
-        paintItem(g, i, yy, (i == selectedItem || i == paintedItem) ? 1 : 0);
-    paintedItem = selectedItem;
-}
-
-int YMenu::findItemPos(int itemNo, int &x, int &y) {
-    x = -1;
-    y = -1;
-    
-    if (itemNo < 0 || itemNo > itemCount())
-        return -1;
-
-    int fontHeight = menuFont->height() + 1;
-    y = 3;
-    x = 2;
-    
-    for (int i = 0; i < itemNo; i++) {
-        if (item(i)->command() == cmdSeparator)
-            y += 4;
-        else {
-            unsigned int h = fontHeight;
-
-            if (item(i)->getPixmap() && item(i)->getPixmap()->height() > h)
-                h = item(i)->getPixmap()->height();
-
-            y += h + 3;
-        }
-    }
-    return 0;
-}
-
-int YMenu::findItem(int x, int y) {
-    if (x < 2 || x > int (width() - 3))
-        return -1;
-
-    int fontHeight = menuFont->height() + 1;
-    int yPos = 2;
-    
-    for (int i = 0; i < itemCount(); i++) {
-        if (item(i)->command() == cmdSeparator) {
-            if (y >= yPos && y < yPos + 4)
-                return i;
-            
-            yPos += 4;
-        } else {
-            unsigned int h = fontHeight;
-
-            if (item(i)->getPixmap() && item(i)->getPixmap()->height() > h)
-                h = item(i)->getPixmap()->height();
-
-            if (y >= yPos && y < int(yPos + h + 3))
-                return i;
-
-            yPos += h + 3;
-        }
-    }
-    return -1;
-}
-
 int YMenu::onCascadeButton(int selItem, int x, int /*y*/) {
     if (selItem != -1 &&
         item(selItem)->command() != cmdSubmenu &&
@@ -386,11 +302,13 @@ void YMenu::focusItem(int itemNo, int submenu, int byMouse) {
 
         if (submenu && sub && item(selectedItem)->isEnabled()) {
             int xp, yp;
-            
+            int l, t, r, b;
+
+            getOffsets(l, t, r, b);
             findItemPos(selectedItem, xp, yp);
-            sub->popup(this, commandListener(),
-                       x() + width() - 2, y() + yp - 3,
-                       width() - 4, -1,
+            sub->popup(this, commandListener(), 0,
+                       x() + width() - r, y() + yp - t,
+                       width() - r - l, -1,
                        YPopupWindow::pfCanFlipHorizontal |
                        (popupFlags() & YPopupWindow::pfFlipHorizontal) |
                        (byMouse ? (unsigned int)YPopupWindow::pfButtonDown : 0U));
@@ -408,9 +326,15 @@ void YMenu::focusItem(int itemNo, int submenu, int byMouse) {
 int YMenu::findActiveItem(int cur, int direction) {
     assert(direction == -1 || direction == 1);
     
-    if (cur == -1 || itemCount() == 0)
+    if (itemCount() == 0)
         return -1;
 
+    if (cur == -1)
+        if (direction == 1)
+            cur = itemCount() - 1;
+        else
+            cur = 0;
+
     assert(cur >= 0 && cur < itemCount());
 
     int c = cur;
@@ -441,7 +365,7 @@ int YMenu::activateItem(int no, int byMouse, unsigned int modifiers) {
 
 bool YMenu::handleKey(const XKeyEvent &key) {
     if (key.type == KeyPress) {
-        int k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+        KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
         int m = KEY_MODMASK(key.state);
 
         if ((m & ~ShiftMask) == 0) {
@@ -470,7 +394,7 @@ bool YMenu::handleKey(const XKeyEvent &key) {
                     for (int i = 0; i < itemCount(); i++) {
                         int hot = item(i)->hotChar();
                         if (hot != -1 &&
-                            TOUPPER(char(hot)) == TOUPPER(k))
+                            TOUPPER(char(hot)) == TOUPPER(char(k)))
                         {
                             focusItem(i, 0, 0);
                             if (!(m & ShiftMask))
@@ -520,13 +444,22 @@ void YMenu::handleButton(const XButtonEvent &button) {
 }
 
 void YMenu::handleMotion(const XMotionEvent &motion) {
-    if ((motion.state & (Button1Mask | Button2Mask | Button3Mask | Button4Mask | Button5Mask))) {
+    bool isButton =
+        (motion.state & (Button1Mask |
+                         Button2Mask |
+                         Button3Mask |
+                         Button4Mask |
+                         Button5Mask)) ? true : false;
+    
+    if (menuMouseTracking || isButton) {
         int selItem = findItem(motion.x_root - x(), motion.y_root - y());
         if (selItem != -1 || app->popup() == this) {
             int submenu = onCascadeButton(selItem,
                                           motion.x_root - x(),
-                                          motion.y_root - y()) ? 0 : 1;
-            focusItem(selItem, submenu, 1);
+                                          motion.y_root - y()) 
+                          && !(motion.state & ControlMask) ? 0 : 1;
+            //if (selItem != -1)
+                focusItem(selItem, submenu, 1);
         }
     }
 
@@ -556,16 +489,17 @@ YMenu::YMenuItem::YMenuItem(const char *name, int aHotCharPos, const char *param
 }
 
 YMenu::YMenuItem::~YMenuItem() {
+    if (fSubmenu && !fSubmenu->isShared())
+        delete fSubmenu;
     delete fName;
     delete fParam;
-    delete fSubmenu;
 }
 
 void YMenu::YMenuItem::setChecked(bool c) {
     fChecked = c;
 }
 
-void YMenu::YMenuItem::setPixmap(YMaskPixmap *pixmap) {
+void YMenu::YMenuItem::setPixmap(YPixmap *pixmap) {
     fPixmap = pixmap;
 }
 
@@ -588,7 +522,7 @@ void YMenu::removeAll() {
     }
     for (int i = 0; i < itemCount(); i++)
         delete fItems[i];
-    free(fItems);
+    FREE(fItems);
     fItems = 0;
     fItemCount = 0;
     paintedItem = selectedItem = -1;
@@ -597,7 +531,7 @@ void YMenu::removeAll() {
 
 YMenu::YMenuItem * YMenu::add(YMenuItem *item) {
     if (item) {
-        YMenuItem **newItems = (YMenuItem **)realloc(fItems, sizeof(YMenuItem *) * ((fItemCount) + 1));
+        YMenuItem **newItems = (YMenuItem **)REALLOC(fItems, sizeof(YMenuItem *) * ((fItemCount) + 1));
         if (newItems) {
             fItems = newItems;
             fItems[fItemCount++] = item;
@@ -635,85 +569,279 @@ void YMenu::disableCommand(WMCommand cmd) {
             item(i)->setEnabled(false);
 }
 
-void YMenu::paint(Graphics &g, int /*_x*/, int /*_y*/, unsigned int /*_width*/, unsigned int /*_height*/) {
-    g.setColor(menuBg);
-    g.drawBorderW(0, 0, width() - 1, height() - 1, true);
-    g.drawLine(1, 1, width() - 3, 1);
-    g.drawLine(1, 2, width() - 3, 2);
-    g.drawLine(1, 1, 1, height() - 3);
-    g.drawLine(2, 1, 2, height() - 3);
-    g.drawLine(1, height() - 3, width() - 3, height() - 3);
-    g.drawLine(width() - 3, 1, width() - 3, height() - 3);
+int YMenu::getItemHeight(int itemNo, int &h, int &top, int &bottom, int &pad) {
+    h = top = bottom = pad = 0;
+    
+    if (itemNo < 0 || itemNo > itemCount())
+        return -1;
 
-    int y = 2;
-    for (int i = 0; i < itemCount(); i++)
-        paintItem(g, i, y, 1);
-    paintedItem = selectedItem;
+    if (item(itemNo)->command() == cmdSeparator) {
+        top = 0;
+        bottom = 0;
+        pad = 1;
+        if (wmLook == lookMetal)
+            h = 3;
+        else
+            h = 4;
+    } else {
+        int fontHeight = menuFont->height() + 1;
+        unsigned int ih = fontHeight;
+
+        if (fontHeight < 16)
+            fontHeight = 16;
+
+        if (item(itemNo)->getPixmap() &&
+            item(itemNo)->getPixmap()->height() > ih)
+            ih = item(itemNo)->getPixmap()->height();
+
+        if (wmLook == lookWarp4 || wmLook == lookWin95) {
+            top = bottom = 0;
+            pad = 1;
+        } else if (wmLook == lookMetal) {
+            top = bottom = 1;
+            pad = 1;
+        } else if (wmLook == lookMotif) {
+            top = bottom = 2;
+            pad = 1;
+        } else if (wmLook == lookGtk) {
+            top = bottom = 2;
+            pad = 1;
+        } else {
+            top = 1;
+            bottom = 2;
+            pad = 1;
+        }
+        h = top + pad + ih + pad + bottom;
+    }
+    return 0;
 }
 
-void YMenu::paintItem(Graphics &g, int i, int &y, int paint) {
+void YMenu::getItemWidth(int i, int &iw, int &nw, int &pw) {
+    iw = nw = pw = 0;
+
+    if (item(i)->command() == cmdSeparator) {
+        iw = 0;
+        nw = 0;
+        pw = 0;
+    } else {
+        YPixmap *p = item(i)->getPixmap();
+        if (p)
+            iw = p->height();
+
+        char *name = item(i)->name();
+        if (name)
+            nw = menuFont->textWidth(name);
+
+        char *param = item(i)->param();
+        if (param)
+            pw = menuFont->textWidth(param);
+    }
+}
+
+void YMenu::getOffsets(int &left, int &top, int &right, int &bottom) {
+    if (wmLook == lookMetal) {
+        left = 1;
+        right = 1;
+        top = 2;
+        bottom = 2;
+    } else {
+        left = 2;
+        top = 2;
+        right = 3;
+        bottom = 3;
+    }
+}
+
+void YMenu::getArea(int &x, int &y, int &w, int &h) {
+    getOffsets(x, y, w, h);
+    w = width() - 1 - x - w;
+    h = height() - 1 - y - h;
+}
+
+int YMenu::findItemPos(int itemNo, int &x, int &y) {
+    x = -1;
+    y = -1;
+    
+    if (itemNo < 0 || itemNo > itemCount())
+        return -1;
+
+    int w, h;
+
+    getArea(x, y, w, h);
+    for (int i = 0; i < itemNo; i++) {
+        int ih, top, bottom, pad;
+        
+        getItemHeight(i, ih, top, bottom, pad);
+        y += ih;
+        /*if (item(i)->command() == cmdSeparator)
+            y += 4;
+        else {
+            unsigned int h = fontHeight;
+
+            if (item(i)->getPixmap() && item(i)->getPixmap()->height() > h)
+                h = item(i)->getPixmap()->height();
+
+            if (wmLook == lookGtk)
+                h++;
+            y += h + 3;
+        }*/
+    }
+    return 0;
+}
+
+int YMenu::findItem(int mx, int my) {
+    int x, y, w, h;
+
+    getArea(x, y, w, h);
+    for (int i = 0; i < itemCount(); i++) {
+        int top, bottom, pad;
+
+        getItemHeight(i, h, top, bottom, pad);
+        if (my >= y && my < y + h && mx > 0 && mx < int(width()) - 1)
+            return i;
+
+        y += h;
+    }
+    return -1;
+}
+
+void YMenu::sizePopup() {
+    int width, height;
+    int maxName = 0;
+    int maxParam = 0;
+    int maxIcon = 16;
+    int l, t, r, b;
+    int padx = 1;
+    int left = 1;
+
+    getOffsets(l, t, r, b);
+
+    width = l;
+    height = t;
+
+    for (int i = 0; i < itemCount(); i++) {
+        int ih, top, bottom, pad;
+        
+        getItemHeight(i, ih, top, bottom, pad);
+        height += ih;
+
+        if (pad > padx)
+            padx = pad;
+        if (top > left)
+            left = top;
+
+        int iw, nw, pw;
+        
+        getItemWidth(i, iw, nw, pw);
+
+        if (item(i)->submenu())
+            pw += 2 + ih;
+
+        if (iw > maxIcon)
+            maxIcon = iw;
+        if (nw > maxName)
+            maxName = nw;
+        if (pw > maxParam)
+            maxParam = pw;
+    }
+
+    //maxWidth = maxName + maxParam + (maxParam ? 8 : 0);
+    //width += 3 + maxh + 2 + 2 + maxWidth + 4 + 4;
+
+    namePos = l + left + padx + maxIcon + 2;
+    paramPos = namePos + 2 + maxName + 6;
+    width = paramPos + maxParam + 4 + r;
+    height += b;
+    
+    setSize(width, height);
+}
+
+void YMenu::paintItems() {
+    Graphics &g = getGraphics();
+    int l, t, r, b;
+    getOffsets(l, t, r, b);
+    
+    for (int i = 0; i < itemCount(); i++)
+        paintItem(g, i, l, t, r, (i == selectedItem || i == paintedItem) ? 1 : 0);
+    paintedItem = selectedItem;
+}
+void YMenu::drawSeparator(Graphics &g, int x, int y, int w) {
+    if (wmLook == lookMetal) {
+        g.setColor(menuBg);
+        g.drawLine(x, y + 0, w, y + 0);
+        g.setColor(activeMenuItemBg);
+        g.drawLine(x, y + 1, w, y + 1);;
+        g.setColor(white);
+        g.drawLine(x, y + 2, w, y + 2);;
+        g.drawLine(x, y, x, y + 2);
+        g.setColor(menuBg);
+    } else {
+        //g.setColor(menuBg); // ASSUMED
+        g.drawLine(x, y + 0, w, y + 0);
+        g.setColor(menuBg->darker());
+        g.drawLine(x, y + 1, w, y + 1);
+        g.setColor(menuBg->brighter());
+        g.drawLine(x, y + 2, w, y + 2);
+        g.setColor(menuBg);
+        g.drawLine(x, y + 3, w, y + 3); y++;
+    }
+}
+
+void YMenu::paintItem(Graphics &g, int i, int &l, int &t, int &r, int paint) {
     int fontHeight = menuFont->height() + 1;
     int fontBaseLine = menuFont->ascent();
     YMenuItem *mitem = item(i);
     char *name = mitem->name();
     char *param = mitem->param();
 
-    //if (wmLook == lookMotif) {
-    //   topGC = brightMenuGC;
-    //    bottomGC = darkMenuGC;
-    //} else {
-    //    bottomGC = brightMenuGC;
-    //    topGC = darkMenuGC;
-    //}
     g.setColor(menuBg);
     if (mitem->command() == cmdSeparator) {
-        if (paint) {
-            g.drawLine(1, y, width() - 3, y);
-            g.setColor(menuBg->darker());
-            g.drawLine(1, y + 1, width() - 3, y + 1);
-            g.setColor(menuBg->brighter());
-            g.drawLine(1, y + 2, width() - 3, y + 2);
-            g.setColor(menuBg);
-            g.drawLine(1, y + 3, width() - 3, y + 3);
-        }
-        y += 4;
+        if (paint)
+            drawSeparator(g, 1, t, width() - 2);
+        t += (wmLook == lookMetal) ? 3 : 4;
     } else {
-        unsigned int hh = fontHeight;
-        
-        if (item(i)->getPixmap() && item(i)->getPixmap()->height() > hh)
-            hh = item(i)->getPixmap()->height();
+        int eh, top, bottom, pad, ih;
 
-        hh++;
+        getItemHeight(i, eh, top, bottom, pad);
+        ih = eh - top - bottom - pad - pad;
+        
         if (i == selectedItem)
             g.setColor(activeMenuItemBg);
+        
         if (paint) {
-            if (wmLook == lookWin95 || wmLook == lookWarp4) {
-                g.fillRect(2, y, width() - 5, hh + 2);
-            } else {
-                g.fillRect(2, y, width() - 5, hh + 2);
-                
-                if (i == selectedItem) {
-//#define NEWMOTIF
-#ifdef NEWMOTIF
-                    bool raised = false;
-#else
-                    bool raised = (wmLook != lookMotif) ? false : true;
-#endif
-                    g.setColor(menuBg);
-                    g.draw3DRect(2, y, width() - 6, hh + 1, raised);
+            g.fillRect(l, t, width() - r - l, eh);
 
-                    if (wmLook == lookMotif) {
-                        g.draw3DRect(3, y + 1, width() - 6 - 2, hh - 1, raised);
-                        /*XDrawLine(app->display(), handle(), topGC,
-                                  4, y + 1, width() - 5, y + 1);
-                        XDrawLine(app->display(), handle(), bottomGC,
-                                  4, 0 + y + hh - 2, width() - 5, 0 + y + hh - 2);
-                        XDrawLine(app->display(), handle(), topGC,
-                                  4, y + 1, 4, 0 + y + hh - 2);
-                        XDrawLine(app->display(), handle(), bottomGC,
-                                  width() - 5, y + 1, width() - 5, 0 + y + hh - 2);*/
-                    }
-                }
+            
+            if (wmLook == lookMetal && i != selectedItem) {
+                g.setColor(white);
+                g.drawLine(1, t, 1, t + eh - 1);
+                g.setColor(menuBg);
+            }
+
+            if (wmLook != lookWin95 && wmLook != lookWarp4 &&
+                i == selectedItem)
+            {
+                bool raised = false;
+#ifdef OLDMOTIF
+                if (wmLook == lookMotif)
+                    raised = true;
+#endif
+
+                g.setColor(menuBg);
+                if (wmLook == lookGtk)
+                    g.drawBorderW(l, t, width() - r - l - 1, eh - 1, true);
+                else if (wmLook == lookMetal) {
+                    g.setColor(activeMenuItemBg->darker());
+                    g.drawLine(l, t, width() - r - l, t);
+                    g.setColor(activeMenuItemBg->brighter());
+                    g.drawLine(l, t + eh - 1, width() - r - l, t + eh - 1);
+                } else
+                    g.draw3DRect(l, t, width() - r - l - 1, eh - 1, raised);
+                    
+
+                if (wmLook == lookMotif)
+                    g.draw3DRect(l + 1, t + 1,
+                                 width() - r - l - 3, eh - 3, raised);
             }
 
             YColor *fg;
@@ -727,17 +855,21 @@ void YMenu::paintItem(Graphics &g, int i, int &y, int paint) {
             g.setFont(menuFont);
             
             int delta = (i == selectedItem) ? 1 : 0;
-            if (wmLook == lookMotif || wmLook == lookWarp4 || wmLook == lookWin95)
+            if (wmLook == lookMotif || wmLook == lookGtk ||
+                wmLook == lookWarp4 || wmLook == lookWin95 ||
+                wmLook == lookMetal)
                 delta = 0;
+            int baseLine = t + top + pad + (ih - fontHeight) / 2 + fontBaseLine + delta;
+                //1 + 1 + t + (eh - fontHeight) / 2 + fontBaseLine + delta;
 
             if (mitem->isChecked()) {
                 XPoint points[4];
 
-                points[0].x = delta + (fontHeight - 10) / 2 + 3;
+                points[0].x = l + 3 + (16 - 10) / 2;
                 points[1].x = 5;
                 points[2].x = 5;
                 points[3].x = -5;
-                points[0].y = y + delta + (fontHeight - 10) / 2 + 4;
+                points[0].y = t + top + pad + ih / 2;
                 points[1].y = -5;
                 points[2].y = 5;
                 points[3].y = 5;
@@ -745,32 +877,28 @@ void YMenu::paintItem(Graphics &g, int i, int &y, int paint) {
                 g.fillPolygon(points, 4, Convex, CoordModePrevious);
             } else {
                 if (mitem->getPixmap())
-                    g.drawMaskPixmap(mitem->getPixmap(),
-                                     2 + delta + 1 + (wmLook == lookMotif ? 1 : 0),
-                                     y + delta + 1 + (hh - mitem->getPixmap()->height()) / 2);
+                    g.drawPixmap(mitem->getPixmap(),
+                                 l + 1 + delta,
+                                 t + delta + top + pad + (eh - top - pad * 2 - bottom - mitem->getPixmap()->height()) / 2);
             }
 
             if (name) {
                 if (!mitem->isEnabled()) {
                     g.setColor(white);
                     g.drawChars((char *)name, 0, strlen((char *)name),
-                                1 + 3 + hh + 4 + delta,
-                                1 + 1 + y + (hh - fontHeight) / 2 + fontBaseLine + delta);
+                                1 + delta + namePos, 1 + baseLine);
                     
                     if (mitem->hotCharPos() != -1) {
-                        g.drawCharUnderline(1 + 2 + hh + 4 + delta,
-                                            1 + 1 + y + (hh - fontHeight) / 2 + fontBaseLine + delta,
+                        g.drawCharUnderline(1 + delta +  namePos, 1 + baseLine,
                                             (char *)name, mitem->hotCharPos());
                     }
                 }
                 g.setColor(fg);
                 g.drawChars((char *)name, 0, strlen((char *)name),
-                            2 + hh + 4 + delta,
-                            1 + y + (hh - fontHeight) / 2 + fontBaseLine + delta);
+                            delta + namePos, baseLine);
 
                 if (mitem->hotCharPos() != -1) {
-                    g.drawCharUnderline(3 + hh + 4 + delta,
-                                        1 + y + (hh - fontHeight) / 2 + fontBaseLine + delta,
+                    g.drawCharUnderline(delta + namePos, baseLine, 
                                         (char *)name, mitem->hotCharPos());
                 }
             }
@@ -779,42 +907,73 @@ void YMenu::paintItem(Graphics &g, int i, int &y, int paint) {
                 if (!mitem->isEnabled()) {
                     g.setColor(white);
                     g.drawChars((char *)param, 0, strlen((char *)param),
-                                1 + 2 + hh + 4 + 8 + paramPos + delta,
-                                1 + 1 + y + (hh - fontHeight) / 2 + fontBaseLine + delta);
+                                paramPos + delta + 1,
+                                baseLine + 1);
                 }
                 g.setColor(fg);
                 g.drawChars((char *)param, 0, strlen((char *)param),
-                            2 + hh + 4 + 8 + paramPos + delta,
-                            1 + y + (hh - fontHeight) / 2 + fontBaseLine + delta);
+                            paramPos + delta,
+                            baseLine);
             }
             else if (mitem->submenu() || mitem->command() == cmdSubmenu) {
+                int active = ((mitem->command() == cmdSubmenu && i == selectedItem) ||
+                              fPopup == mitem->submenu()) ? 1 : 0;
                 if (mitem->command() != cmdSubmenu) {
-                    int active = (fPopup == mitem->submenu()) ? 1 : 0;
                     g.setColor(menuBg);
-                    g.fillRect(width() - 3 - hh - 3 + 2,
-                               1 + y,
-                               hh - 1,
-                               hh - 1);
-                    g.drawBorderW(width() - 3 - hh - 3 + 1, 1 + y,
-                                  hh - 1, hh - 1,
+                    g.fillRect(width() - r - 1 - ih - pad, t + top + pad, ih, ih);
+                    g.drawBorderW(width() - r - 1 - ih - pad, t + top + pad, ih - 1, ih - 1,
                                   active ? false : true);
-                    delta = active ? 1 : 0;
+                    delta = delta ? active ? 1 : 0 : 0;
                 }
 
-                XPoint points[3];
+                if (wmLook == lookGtk || wmLook == lookMotif) {
+                    int asize = 9;
+                    int ax = delta + width() - r - 1 - asize * 3 / 2;
+                    int ay = delta + t + top + pad + (ih - asize) / 2;
+                    g.setColor(menuBg);
+                    g.drawArrow(0, active ? 1 : -1, ax, ay, asize);
+                } else {
+                    int asize = 9;
+                    int ax = width() - r - 1 - asize;
+                    int ay = delta + t + top + pad + (ih - asize) / 2;
 
-                points[0].x = width() - 3 - hh + 2 + delta + 2;
-                points[2].x = - (hh / 2 - 2);
-                points[1].x = hh / 2 - 2;
-                points[0].y = 1 + y + delta + 1;
-                points[1].y = hh / 2 - 2;
-                points[2].y = hh / 2 - 2;
-
-                g.setColor(fg);
-                g.fillPolygon(points, 3, Convex, CoordModePrevious);
+                    if (mitem->submenu() && mitem->command() != cmdSubmenu)
+                        g.setColor(menuItemFg);
+                    else 
+                        g.setColor(fg);
+                    g.drawArrow(0, 0, ax, ay, asize);
+                }
 
             }
         }
-        y += hh + 2;
+        t += eh;
     }
 }
+
+void YMenu::paint(Graphics &g, int /*_x*/, int /*_y*/, unsigned int /*_width*/, unsigned int /*_height*/) {
+    if (wmLook == lookMetal) {
+        g.setColor(activeMenuItemBg);
+        g.drawLine(0, 0, width() - 1, 0);
+        g.drawLine(0, 0, 0, height() - 1);
+        g.drawLine(width() - 1, 0, width() - 1, height() - 1);
+        g.drawLine(0, height() - 1, width() - 1, height() - 1);
+        g.setColor(white);
+        g.drawLine(1, 1, width() - 2, 1);
+        g.setColor(menuBg);
+        g.drawLine(1, height() - 2, width() - 2, height() - 2);
+    } else {
+        g.setColor(menuBg);
+        g.drawBorderW(0, 0, width() - 1, height() - 1, true);
+        g.drawLine(1, 1, width() - 3, 1);
+        g.drawLine(1, 1, 1, height() - 3);
+        g.drawLine(1, height() - 3, width() - 3, height() - 3);
+        g.drawLine(width() - 3, 1, width() - 3, height() - 3);
+    }
+
+    int l, t, r, b;
+    getOffsets(l, t, r, b);
+    
+    for (int i = 0; i < itemCount(); i++)
+        paintItem(g, i, l, t, r, 1);
+    paintedItem = selectedItem;
+}
