@@ -1,14 +1,14 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997 Marko Macek
+ * Copyright (C) 1997,1998 Marko Macek
  */
 
 #include "icewm.h"
 
 #ifdef DEBUG
-int debug = 0;
-int debug_z = 0;
+bool debug = false;
+bool debug_z = false;
 
 void msg(const char *msg, ...) {
     va_list ap;
@@ -37,13 +37,22 @@ void logEvent(XEvent xev) {
             xev.xdestroywindow.window,
             xev.xdestroywindow.event);
         break;
-
+#else
+    case CreateNotify:
+    case DestroyNotify:
+        break;
+#endif
+#if 0
     case MapRequest:
         msg("window=0x%lX: mapRequest parent=0x%lX",
             xev.xmaprequest.window,
             xev.xmaprequest.parent);
         break;
-
+#else
+    case MapRequest:
+        break;
+#endif
+#if 0
     case MapNotify:
         msg("window=0x%lX: mapNotify event=0x%lX, override_redirect=%s",
             xev.xmap.window,
@@ -52,15 +61,13 @@ void logEvent(XEvent xev) {
         break;
 
     case UnmapNotify:
-        msg("window=0x%lX: unmapNotify event=0x%lX, from_configure=%s",
+        msg("window=0x%lX: unmapNotify event=0x%lX, from_configure=%s send_event=%s",
             xev.xunmap.window,
             xev.xunmap.event,
+            xev.xunmap.from_configure ? "True" : "False",
             xev.xunmap.from_configure ? "True" : "False");
         break;
 #else
-    case CreateNotify:
-    case DestroyNotify:
-    case MapRequest:
     case MapNotify:
     case UnmapNotify:
 	break;
@@ -132,14 +139,22 @@ void logEvent(XEvent xev) {
             xev.xvisibility.window,
             xev.xvisibility.state);
         break;
-
+#else
+    case VisibilityNotify:
+        break;
+#endif
+#if 1
     case ClientMessage:
         msg("window=0x%lX: clientMessage message_type=0x%lX format=%d",
             xev.xclient.window,
             xev.xclient.message_type,
             xev.xclient.format);
         break;
-
+#else
+    case ClientMessage:
+        break;
+#endif
+#if 0
     case PropertyNotify:
         msg("window=0x%lX: propertyNotify atom=0x%lX time=%ld state=%d",
             xev.xproperty.window,
@@ -148,8 +163,6 @@ void logEvent(XEvent xev) {
             xev.xproperty.state);
         break;
 #else
-    case VisibilityNotify:
-    case ClientMessage:
     case PropertyNotify:
         break;
 #endif
@@ -189,7 +202,7 @@ void logEvent(XEvent xev) {
     case MotionNotify:
         break;
 #endif
-#if 1
+#if 0
     case EnterNotify:
     case LeaveNotify:
         msg("window=0x%lX: %s root=0x%lX, subwindow=0x%lX, time=%ld, (%d:%d %d:%d) mode=%d detail=%d same_screen=%s, focus=%s state=0x%X",
@@ -239,7 +252,6 @@ void logEvent(XEvent xev) {
     case Expose:
         break;
 #endif
-        
 #if 0
     default:
         msg("window=0x%lX: unknown type=%d", xev.xany.window, xev.type);
@@ -258,3 +270,36 @@ void die(int exitcode, const char *msg, ...) {
     fprintf(stderr, "\n");
     exit(exitcode);
 }
+
+void warn(const char *msg, ...) {
+    va_list ap;
+
+    va_start(ap, msg);
+    vfprintf(stderr, msg, ap);
+    va_end(ap);
+    fprintf(stderr, "\n");
+}
+
+void *MALLOC(size_t len) {
+    if (len == 0)
+        return NULL;
+    return malloc(len);
+}
+
+void *REALLOC(void *p, size_t new_len) {
+    if (p) {
+        if (new_len > 0)
+            return realloc(p, new_len);
+        else {
+            FREE(p);
+            return NULL;
+        }
+    } else {
+        return MALLOC(new_len);
+    }
+}
+
+void FREE(void *p) {
+    if (p)
+        free(p);
+}
