@@ -1,7 +1,7 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997 Marko Macek
+ * Copyright (C) 1997,1998 Marko Macek
  */
 
 #include "icewm.h"
@@ -13,14 +13,12 @@ void YFrameWindow::drawOutline(int x, int y, int w, int h) {
     y += bw / 2;
     w -= bw;
     h -= bw;
-    XDrawRectangle(app->display(),
-                   app->root()->handle(),
-                   outlineGC,
+    XDrawRectangle(app->display(), manager->handle(), outlineGC,
                    x, y, w, h);
 }
 
 int YFrameWindow::handleMoveKeys(const XKeyEvent &key, int &newX, int &newY) {
-    int k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+    KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
     int m = KEY_MODMASK(key.state);
     int factor = 1;
 
@@ -38,13 +36,13 @@ int YFrameWindow::handleMoveKeys(const XKeyEvent &key, int &newX, int &newY) {
     else if (k == XK_Down)
         newY += factor;
     else if (k == XK_Home)
-        newX = app->minX() - borderX();
+        newX = manager->minX(getLayer()) - borderX();
     else if (k == XK_End)
-        newX = app->maxX() - width() + borderX();
+        newX = manager->maxX(getLayer()) - width() + borderX();
     else if (k == XK_Prior)
-        newY = app->minY() - borderY();
+        newY = manager->minY(getLayer()) - borderY();
     else if (k == XK_Next)
-        newY = app->maxY() - height() + borderY();
+        newY = manager->maxY(getLayer()) - height() + borderY();
     else if (k == XK_Return)
         return -1;
     else if (k ==  XK_Escape) {
@@ -60,7 +58,7 @@ int YFrameWindow::handleResizeKeys(const XKeyEvent &key,
                                    int &newX, int &newY, int &newWidth, int &newHeight,
                                    int incX, int incY)
 {
-    int k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+    KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
     int m = KEY_MODMASK(key.state);
     int factor = 1;
 
@@ -132,26 +130,26 @@ void YFrameWindow::handleMoveMouse(const XMotionEvent &motion, int &newX, int &n
     newX += borderX();
     newY += borderY();
     int n = -2;
-    if (newX + int(width() + n * borderX()) > app->maxX())
-        if (newX + int(width() + n * borderX()) < int(app->maxX() + EdgeResistance))
-            newX = app->maxX() - width() - n * borderX();
-//        else
-//            newX -= EdgeResistance;
-    if (newY + int(height() + n * borderY()) > app->maxY())
-        if (newY + int(height() + n * borderY()) < int(app->maxY() + EdgeResistance))
-            newY = app->maxY() - height() - n * borderY();
-//        else
-//            newY -= EdgeResistance;
-    if (newX < app->minX())
-        if (newX > int(- EdgeResistance + app->minX()))
-            newX = app->minX();
-//        else
-//            newX += EdgeResistance;
-    if (newY < app->minY())
-        if (newY > int(- EdgeResistance + app->minY()))
-            newY = app->minY();
-//        else
-//            newY += EdgeResistance;
+    if (newX + int(width() + n * borderX()) > manager->maxX(getLayer()))
+        if (newX + int(width() + n * borderX()) < int(manager->maxX(getLayer()) + EdgeResistance))
+            newX = manager->maxX(getLayer()) - width() - n * borderX();
+        else if (motion.state & ControlMask)
+            newX -= EdgeResistance;
+    if (newY + int(height() + n * borderY()) > manager->maxY(getLayer()))
+        if (newY + int(height() + n * borderY()) < int(manager->maxY(getLayer()) + EdgeResistance))
+            newY = manager->maxY(getLayer()) - height() - n * borderY();
+        else if (motion.state & ControlMask)
+            newY -= EdgeResistance;
+    if (newX < manager->minX(getLayer()))
+        if (newX > int(- EdgeResistance + manager->minX(getLayer())))
+            newX = manager->minX(getLayer());
+        else if (motion.state & ControlMask)
+            newX += EdgeResistance;
+    if (newY < manager->minY(getLayer()))
+        if (newY > int(- EdgeResistance + manager->minY(getLayer())))
+            newY = manager->minY(getLayer());
+        else if (motion.state & ControlMask)
+            newY += EdgeResistance;
     newX -= borderX();
     newY -= borderY();
 }
@@ -333,7 +331,6 @@ end:
 
 void YFrameWindow::manualPlace() {
     int xx = x(), yy = y();
-    int rc;
     Cursor grabPointer = movePointer;
 
     grabX = borderX();
@@ -345,7 +342,7 @@ void YFrameWindow::manualPlace() {
     buttonDownX = 0;
     buttonDownY = 0;
 
-    if (!app->grabEvents(app->root(), grabPointer,
+    if (!app->grabEvents(desktop, grabPointer,
                          ButtonPressMask |
                          ButtonReleaseMask |
                          PointerMotionMask))
@@ -475,19 +472,17 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
                 endMoveSize();
                 break;
             }
-        } else {
-            int k = XKeycodeToKeysym(app->display(), key.keycode, 0);
-            int m = KEY_MODMASK(key.state);
+        } else if (app->AltMask != 0) {
+            KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+            unsigned int m = KEY_MODMASK(key.state);
             
-            if (m == Mod1Mask + ShiftMask) {
+            if (m == app->AltMask + ShiftMask) {
                 if (k == XK_F6)
                     wmPrevWindow();
-                else if (k == XK_Tab)
-                    switchWindow->begin(0);
-            } else if (m == Mod1Mask) {
-                if (k == XK_Tab)
-                    switchWindow->begin(1);
-                else if (k == ' ')
+                else if (k == XK_F10)
+                    wmMaximizeVert();
+            } else if (m == app->AltMask) {
+                if (k == ' ')
                     popupSystemMenu();
                 else if (k == XK_F1)
                     wmRaise();
@@ -513,6 +508,11 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
                     wmHide();
                 else if (k == XK_F12)
                     wmRollup();
+            } else if (isIconic()) {
+                if (k == XK_Return || k == XK_KP_Enter)
+                    wmRestore();
+                else if ((k == XK_Menu) || (k == XK_F10 && m == ShiftMask))
+                    popupSystemMenu();
             }
         }
     }
@@ -520,14 +520,14 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
 }
 
 void YFrameWindow::constrainPositionByModifier(int &x, int &y, const XMotionEvent &motion) {
-    int mask = motion.state & (ShiftMask | Mod1Mask);
+    unsigned int mask = motion.state & (ShiftMask | app->AltMask);
 
     x += borderX();
     y += borderY();
     if (mask == ShiftMask) {
         x = x / 4 * 4;
         y = y / 4 * 4;
-    } else if (mask == Mod1Mask) {
+    } else if (mask == app->AltMask) {
         x = x / 8 * 8;
         y = y / 8 * 8;
     }
@@ -536,25 +536,28 @@ void YFrameWindow::constrainPositionByModifier(int &x, int &y, const XMotionEven
 }
 
 void YFrameWindow::constrainMouseToWorkspace(int &x, int &y) {
-    if (x < app->minX())
-        x = app->minX();
-    else if (x >= app->maxX())
-        x = app->maxX() - 1;
-    if (y < app->minY())
-        y = app->minY();
-    else if (y >= app->maxY())
-        y = app->maxY() - 1;
+    if (x < manager->minX(getLayer()))
+        x = manager->minX(getLayer());
+    else if (x >= manager->maxX(getLayer()))
+        x = manager->maxX(getLayer()) - 1;
+    if (y < manager->minY(getLayer()))
+        y = manager->minY(getLayer());
+    else if (y >= manager->maxY(getLayer()))
+        y = manager->maxY(getLayer()) - 1;
 }
 
-bool YFrameWindow::canSize() {
-    if (style & fsRolledup)
-        return false;
-    if (maximized() && !sizeMaximized)
+bool YFrameWindow::canSize(bool horiz, bool vert) {
+    if (isRollup())
         return false;
 #ifndef NO_MWM_HINTS
     if (!(client()->mwmFunctions() & MWM_FUNC_RESIZE))
         return false;
 #endif
+    if (!sizeMaximized) {
+        if ((!vert || isMaximizedVert()) &&
+            (!horiz || isMaximizedHoriz()))
+            return false;
+    }
     return true;
 }
 
@@ -653,6 +656,8 @@ void YFrameWindow::endMoveSize() {
     statusMoveSize->end();
     movingWindow = 0;
     sizingWindow = 0;
+    if (getLayer() == WinLayerDock)
+        manager->updateWorkArea();
 }
         
 void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent &motion) {
@@ -691,18 +696,17 @@ void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent
 }
 
 void YFrameWindow::moveWindow(int newX, int newY) {
+    if (newX >= int(manager->maxX(getLayer()) - borderX()))
+        newX = manager->maxX(getLayer()) - borderX();
 
-    if (newX >= int(app->maxX() - borderX()))
-        newX = app->maxX() - borderX();
-
-    if (newY >= int(app->maxY() - borderY() - titleY()))
-        newY = app->maxY() - borderY() - titleY();
+    if (newY >= int(manager->maxY(getLayer()) - borderY() - titleY()))
+        newY = manager->maxY(getLayer()) - borderY() - titleY();
     
-    if (newX < int(app->minX() - (width() - borderX())))
-        newX = app->minX() - (width() - borderX());
+    if (newX < int(manager->minX(getLayer()) - (width() - borderX())))
+        newX = manager->minX(getLayer()) - (width() - borderX());
 
-    if (newY < int(app->minY() - (height() - borderY())))
-        newY = app->minY() - (height() - borderY());
+    if (newY < int(manager->minY(getLayer()) - (height() - borderY())))
+        newY = manager->minY(getLayer()) - (height() - borderY());
 
     setPosition(newX, newY);
     
