@@ -13,23 +13,33 @@ YWindow::YWindow(YWindow *parent, Window win) {
     flags = 0;
     fX = fY = 0;
     fWidth = fHeight = 1;
-    fPointer = leftPointer;
+    fPointer = None;
     fStyle = 0;
     fGraphics = 0;
     fEnabled = true;
+    fToplevel = false;
     fFocusedWindow = 0;
     unmapCount = 0;
     accel = 0;
-    fToplevel = 0;
+#ifdef CONFIG_TOOLTIP
     fToolTip = 0;
+#endif
+    fDND = false;
+    XdndDragSource = None;
+    XdndDropTarget = None;
+    fEventMask =
+        KeyPressMask | KeyReleaseMask |
+        LeaveWindowMask | EnterWindowMask | FocusChangeMask;
 
-    insertWindow();
-    
-    if (win != 0) {
-        flags |= wfCreated | wfAdopted;
-        
+    if (fHandle != 0) {
+        MSG(("adopting window %lX\n", fHandle));
+        flags |= wfAdopted;
         create();
+    } else if (fParentWindow == 0) {
+        assert(desktop != 0);
+        fParentWindow = desktop;
     }
+    insertWindow();
 }
 
 YWindow::~YWindow() {
@@ -39,13 +49,16 @@ YWindow::~YWindow() {
         delete accel;
         accel = next;
     }
+#ifdef CONFIG_TOOLTIP
     if (fToolTip) {
         fToolTip->hide();
         if (fToolTipTimer && fToolTipTimer->getListener() == fToolTip) {
             fToolTipTimer->stopTimer();
             fToolTipTimer->setListener(0);
         }
+        delete fToolTip;
     }
+#endif
     if (fGraphics) {
         delete fGraphics;
         fGraphics = 0;
@@ -61,8 +74,19 @@ void YWindow::setFocus() {
                    CurrentTime);
 }
 
-void YWindow::setStyle(int aStyle) {
-    fStyle = aStyle;
+void YWindow::setStyle(unsigned long aStyle) {
+    if (fStyle != aStyle) {
+        fStyle = aStyle;
+
+        if (flags & wfCreated) {
+            if (aStyle & wsManager)
+                fEventMask |=
+                    SubstructureRedirectMask | SubstructureNotifyMask |
+                    ButtonPressMask | ButtonReleaseMask | ButtonMotionMask;
+
+            XSelectInput(app->display(), fHandle, fEventMask);
+        }
+    }
 }
 
 Graphics &YWindow::getGraphics() {
@@ -88,30 +112,35 @@ void YWindow::repaintFocus() {
 }
 
 void YWindow::create() {
-    if (!(flags & wfCreated)) {
-        flags |= wfCreated;
-
+    if (flags & wfCreated)
+        return ;
+    if (fHandle == 0) {
         XSetWindowAttributes attributes;
-        unsigned long eventmask = /*CWBackPixel |*/ CWEventMask | CWCursor;
+        unsigned int attrmask = CWEventMask;
 
-        attributes.cursor = fPointer;
-       
-        attributes.event_mask =
-	    ExposureMask | StructureNotifyMask | 
-            SubstructureRedirectMask | //SubstructureNotifyMask |
-	    LeaveWindowMask | EnterWindowMask | FocusChangeMask |
-            KeyPressMask | KeyReleaseMask |
+        fEventMask |=
+            ExposureMask |
             ButtonPressMask | ButtonReleaseMask | ButtonMotionMask;
-
+        if (fParentWindow == desktop && !(fStyle & wsOverrideRedirect))
+            fEventMask |= StructureNotifyMask;
+        if (fStyle & wsManager)
+            fEventMask |=
+                SubstructureRedirectMask | SubstructureNotifyMask |
+                ButtonPressMask | ButtonReleaseMask | ButtonMotionMask;
+        
         if (fStyle & wsSaveUnder) {
             attributes.save_under = True;
-            eventmask |= CWSaveUnder;
+            attrmask |= CWSaveUnder;
         }
         if (fStyle & wsOverrideRedirect) {
             attributes.override_redirect = True;
-            eventmask |= CWOverrideRedirect;
+            attrmask |= CWOverrideRedirect;
         }
-
+        if (fPointer != None) {
+            attrmask |= CWCursor;
+            attributes.cursor = fPointer;
+        }
+        attributes.event_mask = fEventMask;
         fHandle = XCreateWindow(app->display(),
                                 parent()->handle(),
                                 x(), y(), width(), height(),
@@ -119,10 +148,10 @@ void YWindow::create() {
                                 CopyFromParent,
                                 InputOutput,
                                 CopyFromParent,
-                                eventmask,
+                                attrmask,
                                 &attributes);
 
-        MSG(("window created: 0x%lX at (%d:%d-%dx%d)", fHandle, x(), y(), width(), height()));
+        MSG(("window created: 0x%lX at (%d:%d %dx%d)", fHandle, x(), y(), width(), height()));
         if (flags & wfVisible)
             XMapWindow(app->display(), fHandle);
     } else {
@@ -131,41 +160,38 @@ void YWindow::create() {
         XGetWindowAttributes(app->display(),
                              fHandle,
                              &attributes);
-
         fX = attributes.x;
         fY = attributes.y;
         fWidth = attributes.width;
         fHeight = attributes.height;
 
+        MSG(("window initial geometry (%d:%d %dx%d)",
+             fX, fY, fWidth, fHeight));
+
         if (attributes.map_state != IsUnmapped)
             flags |= wfVisible;
         else
             flags &= ~wfVisible;
 
-        if (parent() == 0) {
-            XSelectInput(app->display(), fHandle,
-                         SubstructureRedirectMask | 
-                         SubstructureNotifyMask |
-                         FocusChangeMask |
-                         PropertyChangeMask |
-                         LeaveWindowMask| EnterWindowMask | 
-                         KeyPressMask | KeyReleaseMask |
-                         ButtonPressMask | ButtonReleaseMask | ButtonMotionMask);
-        } else {
-            XSelectInput(app->display(), fHandle,
-                         StructureNotifyMask | FocusChangeMask |
-                         LeaveWindowMask | EnterWindowMask |
-                         KeyPressMask | KeyReleaseMask |
-                         PropertyChangeMask | ColormapChangeMask
-                        );
-        }
+        fEventMask |=
+            StructureNotifyMask |
+            ColormapChangeMask |
+            PropertyChangeMask;
+
+        if (fStyle & wsManager)
+            fEventMask |=
+                SubstructureRedirectMask | SubstructureNotifyMask |
+                ButtonPressMask | ButtonReleaseMask | ButtonMotionMask;
+        
+        XSelectInput(app->display(), fHandle, fEventMask);
     }
     XSaveContext(app->display(), fHandle, windowContext, (XPointer)this);
+    flags |= wfCreated;
 }
 
 void YWindow::destroy() {
-    XSaveContext(app->display(), fHandle, windowContext, 0);
     if (flags & wfCreated) {
+        XDeleteContext(app->display(), fHandle, windowContext);
         if (!(flags & wfDestroyed)) {
             if (!(flags & wfAdopted)) {
                 XDestroyWindow(app->display(), fHandle);
@@ -271,9 +297,11 @@ void YWindow::handleEvent(const XEvent &event) {
         break;
     case MotionNotify: 
          {
-             XEvent new_event;
+             XEvent new_event, old_event;
 
-             if (XCheckMaskEvent(app->display(),
+             old_event = event;
+             while (XPending(app->display()) > 0 &&
+                    XCheckMaskEvent(app->display(),
                                  KeyPressMask |
                                  KeyReleaseMask |
                                  ButtonPressMask | 
@@ -284,10 +312,12 @@ void YWindow::handleEvent(const XEvent &event) {
                  if (event.type != new_event.type) {
                     XPutBackEvent(app->display(), &new_event);
                     handleMotion(event.xmotion);
-                 } else
-                    handleMotion(new_event.xmotion);
-             } else 
-                 handleMotion(event.xmotion);
+                    return ;
+                 } else {
+                     old_event = new_event;
+                 }
+             }
+             handleMotion(old_event.xmotion);
          } 
          break;
 
@@ -298,7 +328,9 @@ void YWindow::handleEvent(const XEvent &event) {
     case PropertyNotify: handleProperty(event.xproperty); break;
     case ColormapNotify: handleColormap(event.xcolormap); break;
     case MapRequest: handleMapRequest(event.xmaprequest); break;
-    case ConfigureRequest: handleConfigureRequest(event.xconfigurerequest); break;
+    case ConfigureRequest:
+        handleConfigureRequest(event.xconfigurerequest);
+        break;
     case ConfigureNotify: handleConfigure(event.xconfigure); break;
     case DestroyNotify: handleDestroyWindow(event.xdestroywindow); break;
     case Expose: handleExpose(event.xexpose); break;
@@ -333,6 +365,7 @@ void YWindow::handleExpose(const XExposeEvent &expose) {
           expose.height);
 
     XSetClipMask(app->display(), g.handle(), None);
+    //XFlush(app->display());
 }
 
 void YWindow::handleGraphicsExpose(const XGraphicsExposeEvent &graphicsExpose) {
@@ -355,24 +388,25 @@ void YWindow::handleGraphicsExpose(const XGraphicsExposeEvent &graphicsExpose) {
     XSetClipMask(app->display(), g.handle(), None);
 }
 
-void YWindow::handleConfigure(const XConfigureEvent &/*configure*/) {
-    /*if (configure.x != fX ||
-        configure.y != fY ||
-        (unsigned int)configure.width != fWidth ||
-        (unsigned int)configure.height != fHeight)
-    {
-        fX = configure.x;
-        fY = configure.y;
-        fWidth = configure.width;
-        fHeight = configure.height;
-
-        this->configure(fX, fY, fWidth, fHeight);
-    }*/
+void YWindow::handleConfigure(const XConfigureEvent &configure) {
+    if (configure.window == handle())
+        if (configure.x != fX ||
+            configure.y != fY ||
+            (unsigned int)configure.width != fWidth ||
+            (unsigned int)configure.height != fHeight)
+        {
+            fX = configure.x;
+            fY = configure.y;
+            fWidth = configure.width;
+            fHeight = configure.height;
+            
+            this->configure(fX, fY, fWidth, fHeight);
+        }
 }
 
 bool YWindow::handleKey(const XKeyEvent &key) {
     if (key.type == KeyPress) {
-        unsigned int k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+        KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
         unsigned int m = KEY_MODMASK(key.state);
 
         if (accel) {
@@ -460,6 +494,7 @@ void YWindow::handleButton(const XButtonEvent &button) {
                 handleEndDrag(fClickEvent, button);
         }
     }
+#ifdef CONFIG_TOOLTIP
     if (fToolTip) {
         fToolTip->hide();
         if (fToolTipTimer && fToolTipTimer->getListener() == fToolTip) {
@@ -467,6 +502,7 @@ void YWindow::handleButton(const XButtonEvent &button) {
             fToolTipTimer->setListener(0);
         }
     }
+#endif
 }
 
 void YWindow::handleMotion(const XMotionEvent &motion) {
@@ -481,7 +517,8 @@ void YWindow::handleMotion(const XMotionEvent &motion) {
                 ((x_dif < 0) ? - x_dif : x_dif) +
                 ((y_dif < 0) ? - y_dif : y_dif);
             
-            if (motionDelta >= ClickMotionDistance) {
+            if ((motion.time - fClickTime > ClickMotionDelay) || // !!! test
+                (motionDelta >= ClickMotionDistance)) {
                 fClickDrag = 1;
                 handleBeginDrag(fClickEvent, motion);
             }
@@ -489,25 +526,34 @@ void YWindow::handleMotion(const XMotionEvent &motion) {
     }
 }
 
+#ifdef CONFIG_TOOLTIP
 YTimer *YWindow::fToolTipTimer = 0;
+#endif
 
 void YWindow::setToolTip(const char *tip) {
+#ifdef CONFIG_TOOLTIP
     if (fToolTip) {
         fToolTip->hide();
         if (fToolTipTimer && fToolTipTimer->getListener() == fToolTip) {
             fToolTipTimer->stopTimer();
             fToolTipTimer->setListener(0);
         }
-        fToolTip = 0;
+        if (!tip) {
+            delete fToolTip;
+            fToolTip = 0;
+        }
     }
     if (tip) {
-        fToolTip = new YToolTip(app->root());
+        if (!fToolTip)
+            fToolTip = new YToolTip();
         if (fToolTip)
             fToolTip->setText(tip);
     }
+#endif
 }
 
 void YWindow::handleCrossing(const XCrossingEvent &crossing) {
+#ifdef CONFIG_TOOLTIP
     if (fToolTip) {
         if (crossing.type == EnterNotify) {
             if (fToolTipTimer == 0)
@@ -526,6 +572,7 @@ void YWindow::handleCrossing(const XCrossingEvent &crossing) {
             }
         }
     }
+#endif
 }
 
 void YWindow::handleProperty(const XPropertyEvent &) {
@@ -543,6 +590,14 @@ void YWindow::handleClientMessage(const XClientMessageEvent &message) {
         && message.data.l[0] == (long)_XA_WM_DELETE_WINDOW)
     {
         handleClose();
+    } else if (message.message_type == XA_XdndEnter ||
+               message.message_type == XA_XdndLeave ||
+               message.message_type == XA_XdndPosition ||
+               message.message_type == XA_XdndStatus ||
+               message.message_type == XA_XdndDrop ||
+               message.message_type == XA_XdndFinished)
+    {
+        handleXdnd(message);
     }
 }
 
@@ -651,19 +706,19 @@ void YWindow::mapToGlobal(int &x, int &y) {
 
     XTranslateCoordinates(app->display(),
                           handle(),
-                          app->root()->handle(),
+                          desktop->handle(),
                           x, y,
                           &dx, &dy, &child);
     x = dx;
     y = dy;
 }
 
-void YWindow::mapToWindow(int &x, int &y) {
+void YWindow::mapToLocal(int &x, int &y) {
     int dx, dy;
     Window child;
 
     XTranslateCoordinates(app->display(),
-                          app->root()->handle(),
+                          desktop->handle(),
                           handle(),
                           x, y,
                           &dx, &dy, &child);
@@ -691,7 +746,7 @@ void YWindow::setPointer(Cursor pointer) {
 
 void YWindow::setGrabPointer(Cursor pointer) {
     XChangeActivePointerGrab(app->display(),
-                             ButtonPressMask | PointerMotionMask | ButtonReleaseMask, /* ??? */
+                             ButtonPressMask | PointerMotionMask | ButtonReleaseMask,
                              pointer, CurrentTime);//app->getEventTime());
 }
 
@@ -702,9 +757,26 @@ void YWindow::grabKey(int key, unsigned int modifiers) {
                  GrabModeAsync, GrabModeAsync);
         XGrabKey(app->display(), keycode, modifiers | LockMask, handle(), True,
                  GrabModeAsync, GrabModeAsync);
+
+        if (app->NumLockMask != 0) {
+            XGrabKey(app->display(), keycode, modifiers | app->NumLockMask, handle(), True,
+                     GrabModeAsync, GrabModeAsync);
+            XGrabKey(app->display(), keycode, modifiers | LockMask | app->NumLockMask, handle(), True,
+                     GrabModeAsync, GrabModeAsync);
+        }
     }
 }
 
+void YWindow::grabButton(int button, unsigned int modifiers) {
+    // !!! fix Lock+Num
+    XGrabButton(app->display(),
+                button, modifiers,
+                handle(), True,
+                ButtonPressMask | ButtonReleaseMask | ButtonMotionMask,
+                GrabModeAsync, GrabModeAsync,
+                None, None);
+}
+
 void YWindow::captureEvents() {
     app->captureGrabEvents(this);
 }
@@ -842,3 +914,151 @@ void YWindow::removeAccelerator(unsigned int key, unsigned int mod, YWindow *win
     } else parent()->removeAccelerator(key, mod, win);
 }
 
+void YWindow::setWinGravity(int gravity) {
+    unsigned long eventmask = CWWinGravity;
+    XSetWindowAttributes attributes;
+    attributes.win_gravity = gravity;
+
+    XChangeWindowAttributes(app->display(), handle(), eventmask, &attributes);
+}
+
+const Atom XdndCurrentVersion = 3;
+
+void YWindow::setDND(bool enabled) {
+    if (fDND != enabled) {
+        fDND = enabled;
+
+        if (fDND) {
+            XChangeProperty(app->display(), handle(),
+                            XA_XdndAware, XA_ATOM, // ??? ATOM
+                            32, PropModeReplace,
+                            (unsigned char *)&XdndCurrentVersion, 1);
+        } else {
+            XDeleteProperty(app->display(), handle(), XA_XdndAware);
+        }
+    }
+}
+
+void YWindow::XdndStatus(bool acceptDrop, Atom dropAction) {
+    XClientMessageEvent msg;
+    int x_root = 0, y_root = 0;
+
+    mapToGlobal(x_root, y_root);
+    
+    msg.type = ClientMessage;
+    msg.display = app->display();
+    msg.window = XdndDragSource;
+    msg.message_type = XA_XdndStatus;
+    msg.format = 32;
+    msg.data.l[0] = handle();
+    msg.data.l[1] = (acceptDrop ? 0x00000001 : 0x00000000) | 2;
+    msg.data.l[2] = (x_root << 16) + y_root;
+    msg.data.l[3] = (width() << 16) + height();
+    msg.data.l[4] = dropAction;
+    XSendEvent(app->display(), XdndDragSource, False, 0L, (XEvent *)&msg);
+}
+
+void YWindow::handleXdnd(const XClientMessageEvent &message) {
+    if (message.message_type == XA_XdndEnter) {
+        printf("XdndEnter source=%lX\n", message.data.l[0]);
+        XdndDragSource = message.data.l[0];
+    } else if (message.message_type == XA_XdndLeave) {
+        printf("XdndLeave source=%lX\n", message.data.l[0]);
+        if (XdndDropTarget) {
+            YWindow *win;
+
+            if (XFindContext(app->display(), XdndDropTarget, windowContext,
+                             (XPointer *)&win) == 0)
+                win->handleDNDLeave();
+        }
+        XdndDragSource = None;
+    } else if (message.message_type == XA_XdndPosition &&
+               XdndDragSource != 0)
+    {
+        Window target, child;
+        int x, y, nx, ny;
+
+        XdndDragSource = message.data.l[0];
+        x = int(message.data.l[2] >> 16);
+        y = int(message.data.l[2] & 0xFFFF);
+
+        target = handle();
+        
+        printf("XdndPosition source=%lX %d:%d time=%ld action=%ld window=%ld\n",
+                 message.data.l[0],
+                 x, y,
+                 message.data.l[3],
+                 message.data.l[4],
+                 XdndDropTarget);
+
+
+        do {
+            if (XTranslateCoordinates(app->display(),
+                                      desktop->handle(), target,
+                                      x, y, &nx, &ny, &child))
+            {
+                if (child)
+                    target = child;
+            } else {
+                target = 0;
+                break;
+            }
+        } while (child);
+
+        if (target) {
+            if (XdndDropTarget) {
+                YWindow *win;
+                
+                if (XFindContext(app->display(), XdndDropTarget, windowContext,
+                                 (XPointer *)&win) == 0)
+                    win->handleDNDLeave();
+            }
+            XdndDropTarget = target;
+            if (XdndDropTarget) {
+                YWindow *win;
+                
+                if (XFindContext(app->display(), XdndDropTarget, windowContext,
+                                 (XPointer *)&win) == 0)
+                    win->handleDNDEnter();
+            }
+        }
+        printf("XdndPosition %d:%d target=%ld\n",
+               nx, ny, XdndDropTarget);
+        XdndStatus(false, None);
+        /*{
+            XClientMessageEvent msg;
+
+            msg.data.l[0] = handle();
+            msg.data.l[1] = (false ? 0x00000001 : 0x00000000) | 2;
+            msg.data.l[2] = 0; //(x << 16) + y;
+            msg.data.l[3] = 0;//(1 << 16) + 1;
+            msg.data.l[4] = None;
+            XSendEvent (app->display(), XdndDragSource, True, 0L, (XEvent *)&msg);
+        }*/
+    } else if (message.message_type == XA_XdndStatus) {
+        printf("XdndStatus\n");
+    } else if (message.message_type == XA_XdndDrop) {
+        printf("XdndDrop\n");
+    } else if (message.message_type == XA_XdndFinished) {
+        printf("XdndFinished\n");
+    }
+}
+
+void YWindow::handleDNDEnter() {
+}
+
+void YWindow::handleDNDLeave() {
+}
+
+YDesktop::YDesktop(YWindow *aParent = 0, Window win = 0):
+    YWindow(aParent, win)
+{
+}
+YDesktop::~YDesktop() {
+}
+    
+void YDesktop::resetColormap(bool /*active*/) { /// !!!
+}
+
+void YDesktop::resetInputFocus() {
+}
